<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: UploadsController.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Default
 * @copyright  Copyright (c) 2012 VEC
 */

class UploadsController extends Zend_Controller_Action
{

    protected $_maxFileSize = '5MB';

    protected $_allowedExtensions = array(
        'png' , 
        'jpg' ,
        'jpeg' ,
        'gif' ,
        'doc' ,
        'odt' , 
        'xls' ,
        'ods' ,
        'odt' ,
        'pdf'
    );
    
    public function indexAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        $id = intval($this->getRequest()->getParam('id'));
        
        $filesTable = new Giscuit_Db_Table_Files();
        $result = $filesTable->getItems(array(
            'whereId' => $id
        ));

        if(count($result) == 1) {
            $layerId = $result->current()->for_layer_id;
        } else {
            $layerId = NULL;
        }
        
        if(! empty($layerId)) {
            if(! empty($userId)) {
                $table = Giscuit_Db_Table_Layers::getLayerTableById($layerId, array(
                    'withAcl' => 't',
                    'whereActive' => 't' 
                ));
                
                if(! $table) {
                    $result = array();
                }
            } else {
                $table = Giscuit_Db_Table_Layers::getLayerTableById($layerId, array(
                    'withPublish' => 't',
                    'whereActive' => 't' 
                ));
                
                $settings = $table->getSettings();
                
                if(! count($settings['destination'])) {
                    $result = array();
                }
            }
        }
        
        if (count($result) > 0) {
            $result = $result->current();

            //Extension
            $ext = pathinfo($result->name);
            $ext = strtolower($ext['extension']);
            
            if(in_array($ext, Giscuit_Db_Table_Files::$imageExtensions)) {
                if($ext == 'jpeg' || $ext == 'jpg') {
                    $mimeType = 'image/jpeg';   
                } else {
                    $mimeType = 'image/' . $ext; 
                }
                
                header("Content-Type: {$mimeType}");    
            } else {
                header("Content-Disposition: attachment; filename=\"{$result->name}\"");
            }
            
            $db = Zend_Registry::get('db');
            $pdo = $db->getConnection();
            $pdo->beginTransaction();
            
            $data = $pdo->pgsqlLOBOpen($result->oid, 'r');
            
            fpassthru($data);
            
            $pdo->commit();
        
        } else {
            throw new Giscuit_Exception('Invalid file id (' . $id . ')');
        }
    
    }

    public function uploadAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $adapter = new Zend_File_Transfer_Adapter_Http();
        $adapter->setDestination(GT_TMP_PATH);
        
        //Add validators
        $adapter->addValidator('Count', false, 1);
        
        $adapter->addValidator('Size', false, array(
            'min' => 1 , 
            'max' => $this->_maxFileSize
        ), 'file');
        
        $adapter->addValidator('Extension', false, $this->_allowedExtensions, 'file');
        
        $info = $adapter->getFileInfo();
        
        $filesTable = new Giscuit_Db_Table_Files();
        
        if ($adapter->receive()) {
            
            $data = array(
                //'type' => $info['file']['type'] , 
                'file' => $info['file']['destination'] . DIR_SEP . $info['file']['name'],
                'name' => htmlspecialchars($info['file']['name'], ENT_QUOTES)
            );
            
            $filesTable = new Giscuit_Db_Table_Files();
            $id = $filesTable->addItem($data);
            
            $result['status'] = 'UPLOADED';
            $result['url'] = $this->_helper->url->url(array(
                'language' => Zend_Registry::get('Zend_Translate')->getLocale(), 
                'module' => 'default' , 
                'controller' => 'uploads' , 
                'action' => 'index' , 
                'id' => $id
            ), null, true);
        
        } else {
            $messages = array_values($adapter->getMessages());
            $error = $messages[0];
            
            $result['status'] = $error;
        }
                
        //FIXME: Json helper does not work file uploads
        echo Zend_Json::encode($result);
    
    }

}