<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id$
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Mobile
 * @copyright  Copyright (c) 2012 VEC
 */

class Mobile_OfflineController extends Zend_Controller_Action
{
    public $maxFeaturesPerLayer = 2000;
    
    public function manifestAction()
    {
        $config = Zend_Registry::get('config');
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
        $settings = Zend_Registry::get('settings');
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        //Cache
        if(GT_PRODUCTION == false || empty($userId)) {
            header($_SERVER['SERVER_PROTOCOL'] . ' 404 Not Found');
            header('Status: 404 Not Found'); 
            exit;
            
        } else {
            $this->getResponse()->setHeader('Expires', 'Thu, 19 Nov 1981 08:52:00 GMT', true);
            $this->getResponse()->setHeader('Cache-Control', 'no-store, no-cache, must-revalidate, post-check=0, pre-check=0', true);
            $this->getResponse()->setHeader('Pragma', 'no-cache', true);
        }
        
        if(! file_exists(GT_TMP_PATH . DIR_SEP . 'offline_timestamp')) {
            file_put_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp', time());
        }
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $files = array(
            "/{$locale}/mobile/map" ,
            "/{$locale}/default/jsbuild/index/type/mobilemap/version/{$config->version}" ,
        
            "/static-{$config->version}/js/jquery-1.8.2.min.js" ,
            "/static-{$config->version}/js/jquery.mobile-1.2.0.min.js" ,
            "/static-{$config->version}/js/OpenLayers.mobile.js" ,
            "/static-{$config->version}/css/jquery.mobile-1.2.0.min.css" ,
            "/static-{$config->version}/img/icons-18-white.png" ,
            "/static-{$config->version}/img/icons-36-white.png" ,
            "/static-{$config->version}/css/mobile.css" ,
            "/static-{$config->version}/img/ajax-loader.png" ,
            "/static-{$config->version}/img/sprite_mobile.png" ,
            "/static-{$config->version}/img/edit_sprite_mobile.png" ,
            "/static-{$config->version}/img/blank.gif" ,
            "/static-{$config->version}/img/world.png" ,
            "/static-{$config->version}/img/marker/1.png" ,
        );

        $this->getResponse()->setHeader('Content-Type', 'text/cache-manifest');
                    
        echo "CACHE MANIFEST\n";

        echo "CACHE:\n";
        foreach ($files as $file) {
            echo "{$config->httpPath}{$file}\n";
        }

        echo "FALLBACK:\n";
        echo "/ /{$locale}/mobile/map\n";
        
        echo "NETWORK:\n";
        echo "*\n";
        
        echo "# Offline timestamp: " . file_get_contents(GT_TMP_PATH . DIR_SEP . 'offline_timestamp'); 
    }
    
    public function downloadAction() 
    {
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        
        //Id
        $id = intval($request->getParam('id'));
        
        //Where
        $where = $request->getParam('where');
        $where = explode(',', $where);
        $where = array_map('floatval', $where);
        if (count($where) != 4) {
            throw new Giscuit_Db_Table_Exception('Bounding box does not have 4 coordinates');
        }
        $bbox = "ST_SetSRID('BOX3D({$where[0]} {$where[1]}, {$where[2]} {$where[3]})'::box3d, {$config->map->srid})";
        
        //
        $mapOptions = array(
            'orderDirection' => 'asc' , 
            'whereActive' => 't' , 
            'withLayerSettings' => true , 
            'withAcl' => true , 
            'withFields' => true ,
            'withClassesAndStyles' => true
        );

        $layersTable = new Giscuit_Db_Table_Layers();
        $layersSettings = $layersTable->getItems(array_merge(array(
            'whereId' => $id
        ), $mapOptions));

        if(empty($layersSettings)) {
            throw new Giscuit_Db_Table_Exception('Invalid layer id');
        }

        $layerSettings = $layersSettings[$id];
        
        //Class
        $className = 'Giscuit_Map_Layer_' . $layerSettings['class'];
        if (! @class_exists($className)) {
            throw new Giscuit_Exception("Invalid layer class ({$className})");
        }
        
        if(is_subclass_of($className, 'Giscuit_Map_Layer_Vector')) {
            
            //Layer
            $layerClassName = 'Giscuit_Map_Layer_' . $layerSettings['class'];
            $map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
            $map->setParamsFromConfig();
            $map->addEditLayer = true;
            $layer = new $layerClassName($map, $layerSettings);
            
            //Table
            $table = $layer->getTable();
            
            //Form
            $formName = 'Giscuit_Form_Layer_' . $layerSettings['class'];
            $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $layerSettings['class'] . '.xml';
            if (file_exists($configPath)) {
                $configContent = new Zend_Config_Xml($configPath);
                $form = new $formName($configContent);
            } else {
                $form = NULL;
            }
            $form->setMethod('post');
            $form->addAttribs(array(
                'id' => 'gtEditattrForm'
            ));
            $form->multiFix();
            $form->setMethod('post');
            $form->addAttribs(array(
                'id' => 'gtEditattrForm'
            ));
            
            //Legend
            foreach ($layer->children as $legendClass) {
                $legendClass->createLegend();
            }
    
            $legendView = new Zend_View();
            $legendView->addHelperPath(GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'View' . DIR_SEP . 'Helper', 'Giscuit_View_Helper');
            $legendView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'mobile' . DIR_SEP . 
                'views' . DIR_SEP . 'scripts' . DIR_SEP . 'legend');
            $legendView->legend = $map->legendArr;
            $legendView->fetchImage = true;
            $result['legend'] = $legendView->render('index' . GT_SCRIPT_EXT);
    
            //Count
            $select = $table->getSelect('count', array(
                'geometryAsGeoJSON' => true ,
            ));
                    
            $table->addBbox($select, $bbox);
            
            $amount = $table->fetchRow($select)->count;
            
            if($amount <= $this->maxFeaturesPerLayer) {
                $result['records'] = array();
                
                $select = $table->getSelect('all_table', array(
                    'geometryAsGeoJSON' => true ,
                ));
                $table->addBbox($select, $bbox);
                
                //
                $stmt = $table->getAdapter()->query((string) $select);
                while ($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
                    $data = new stdClass();
                    $data->type = 'Feature';
                    $data->geometry = Zend_Json::decode($row['the_geom']);
                    
                    unset($row['the_geom']);
                    $data->properties = $row;
                                    
                    $result['records'][] = $data;
                }
                
                //SLD
                $result['sld'] = $layer->generateSLD();
                
                //Type
                if (class_exists($layerClassName)) {
                    if (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Point')) {
                        $result['geometryType'] = 'Point';
                    } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Line')) {
                        $result['geometryType'] = 'Line';
                    } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Polygon')) {
                        $result['geometryType'] = 'Polygon';
                    }
                }
                
                //FIXME: Check if layer is editable
                //Form
                if($form != NULL) {
                    $formView = new Zend_View();
                    $formView->form = (string) $form;
                    
                    $formView->addScriptPath(GT_MODULES_PATH . DIR_SEP . 'mobile' . DIR_SEP . 
                        'views' . DIR_SEP . 'scripts' . DIR_SEP . 'editgeom');
                    
                    $result['form'] = $formView->render('get' . GT_SCRIPT_EXT);
                }
                    
            } else {
                //TODO:
                $result['error'] = $translate->_('tooManyFeaturesRequested');
            }
        
        } else {
            $tilecacheLayer = md5(uniqid(rand(), true));
            
            $tilecacheCfg = file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg');
            if(! stristr($tilecacheCfg, "\n; Temporary\n")) {
                $tilecacheCfg .= "\n; Temporary\n";
            } 
            
            $tilecacheCfg .= "[{$tilecacheLayer}]\n";
            $tilecacheCfg .= Giscuit_Db_Table_LayerPublish::getTilecacheParams(array(
                'class' => $layerSettings['class'],
                'format' => 'jpeg',
                'action' => 'offline'
            ));
            $tilecacheCfg .= "layers={$layerSettings['class']}\n";
            $tilecacheCfg .= "\n";
            
            file_put_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg', $tilecacheCfg);
            
            $result['tilecache'] = array(
                'tmpname' => $tilecacheLayer,
            );
        }
        
        $result['timestamp'] = time();
        $this->_helper->json($result);
    }
    
    public function downloadtilesAction() 
    {
        $config = Zend_Registry::get('config');
        
        $result['replyCode'] = 200;

        //address
         $address = 'http';
         if (isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on") {
             $address .= "s";
         }
         $address .= "://";
         if ($_SERVER["SERVER_PORT"] != "80") {
              $address .= $_SERVER["SERVER_NAME"] . ":" . $_SERVER["SERVER_PORT"];
         } else {
              $address .= $_SERVER["SERVER_NAME"];
         }
        
        //tmpname
        $tmpname = $this->_request->getParam('tmpname');
        if(! preg_match('/^[a-f0-9]{32}$/', $tmpname)) {
            throw new Giscuit_Exception("Invliad tmpname ({$tmpname})");
        }
        
        //tiles
        $tiles = explode(',', $this->_request->getParam('tiles'));
        
        //FIXME: Hardcoded value of 10
        if(count($tiles) > 10) {
            throw new Giscuit_Exception('Too many tiles requested (' . 
                count($tiles) . ')');
        }
        
        foreach ($tiles as $tile) {
            $url = $address . $config->httpPath . '/tiles/' . $tmpname . '/' . $tile;
            if (filter_var($url, FILTER_VALIDATE_URL) !== false) {
                $contents = @file_get_contents($url);
                if(! empty($contents)) {
                    $data = 'data:image/jpeg;base64,' . base64_encode($contents);
                    $result['records'][$tile] = $data;
                }    
            }
        }
        
        $this->_helper->json($result);
    }
    
    public function downloadtilessuccessAction() 
    {
        $result['replyCode'] = 200;
        
        //tmpname
        $tmpname = $this->_request->getParam('tmpname', '');
        if(! preg_match('/^[a-f0-9]{32}$/', $tmpname)) {
            throw new Giscuit_Exception("Invliad tmpname ({$tmpname})");
        }

        $tilecacheCfg = file_get_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg');
        $tilecacheCfg = explode("\n", $tilecacheCfg);
        
        $section = '';
        foreach ($tilecacheCfg as $key => $value) {
            $value = trim($value);
            
            if(strlen($value) > 2 && 
                $value[0] == '[' && 
                $value[strlen($value) - 1] == ']') {
                    
                $section = rtrim(ltrim($value, '['), ']');
            }
            
            if($section == $tmpname) {
                if(! empty($tilecacheCfg)) {
                    unset($tilecacheCfg[$key]);
                }
            }
        }
        
        $tilecacheCfg = implode("\n", $tilecacheCfg) . "\n";
        file_put_contents(GT_PUBLIC_PATH . DIR_SEP . 'tilecache' . DIR_SEP . 'tilecache.cfg', $tilecacheCfg);
        
        //
        require_once GT_MODULES_PATH . DIR_SEP . 'admin' . DIR_SEP . 'controllers' .
            DIR_SEP . 'LayersController.php';
        
        Admin_LayersController::deleteDirectory(GT_PUBLIC_PATH . DIR_SEP . 'tiles' . 
            DIR_SEP . $tmpname);
        
        $this->_helper->json($result);
    }
}