<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: HistorydataController.php 3010 2012-10-03 12:02:59Z vlad.bruma $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Tool
 * @copyright  Copyright (c) 2012 VEC
 */

class Tool_HistorydataController extends Zend_Controller_Action
{

    public $limit = 10000;
    
    protected $_changed = false;

    protected $_changes = array();

    protected $_errors = array();

    protected $_hisotryTables = array();

    protected $_bounds = array();

    protected $_desc = array();
    
    protected $_map;

    public function init ()
    {
        $this->_desc['shp'] = array(
            'name' => 'ESRI Shapefile' , 
            'extensions' => array(
                'shp' , 
                'shx' , 
                'dbf' , 
                'prj'
            )
        );
        $this->_desc['tab'] = array(
            'name' => 'MapInfo File' , 
            'extensions' => array(
                'dat' , 
                'id' , 
                'map' , 
                'tab'
            )
        );
    }

    public function uploadgetAction ()
    {
        $this->view->form = $this->getUploadForm();
    }

    public function uploadAction ()
    {
        set_time_limit(0);
        
        $translate = Zend_Registry::get('Zend_Translate');
        
        $request = $this->getRequest();
        $fileName = $request->getParam('filename');
        
        //Initialize the map                
        $this->_map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
        $this->_map->setParamsFromConfig();
        $this->_map->setExtent($this->_map->getExtent());
        
        if ($fileName == NULL) {
            $form = $this->getUploadForm();
            $form->populate($request->getParams());
            
            //Invalid input data
            if (! $form->isValid($request->getParams())) {
                $this->view->form = $form;
            } else {
                if ($form->archive->receive()) {
                    
                    $fileName = $form->archive->getFilename(NULL, false);
                    $fileName = substr($fileName, 0, - 4);
                    
                    try {
                        $historyId = $this->upload($fileName);
                    } catch (Giscuit_Exception $e) {
                        $this->view->msg = $translate->_($e->getMessage());
                    }
                    
                    if (! empty($this->_errors)) {
                        $this->view->msg = NULL;
                        $this->view->errors = $this->_errors;
                        $this->view->filename = $fileName;
                    } elseif (! empty($e)) {    
    
                    //Check if an exception has been raised
                    } else {
                        $this->view->msg = $this->uploadsuccess($fileName, $historyId);
                    }
                }
            }
        } else {
            $options = array();
            if ($request->getParam('skip') != NULL) {
                $options['skip'] = explode(',', $request->getParam('skip'));
                $options['skip'] = array_map('intval', $options['skip']);
            }
            if ($request->getParam('overwrite') != NULL) {
                $options['overwrite'] = explode(',', $request->getParam('overwrite'));
                $options['overwrite'] = array_map('intval', $options['overwrite']);
            }
            
            try {
                $historyId = $this->upload($fileName, $options);
            } catch (Giscuit_Exception $e) {
                $this->view->msg = $translate->_($e->getMessage());
            }
            
            if (! empty($this->_errors)) {
                $this->view->msg = NULL;
                $this->view->errors = $this->_errors;
                $this->view->filename = $fileName;
            } elseif (! empty($e)) {    
    
            //Check if an exception has been raised
            } else {     
                $this->view->msg = $this->uploadsuccess($fileName, $historyId);
            }
        }
    }
    
    public function uploadsuccess ($fileName, $historyId)
    {
        //Get current user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $translate = Zend_Registry::get('Zend_Translate');
        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        
        $mainHistory = new Giscuit_Db_Table_History();
                        
        foreach ($this->_hisotryTables as $history) {
            $history->clearHistory($historyId, $userId);
        }
                        
        //Apply add/edit/delete changes 
        foreach ($this->_changes as $row) {
            try {
                $id = $row['object']->save();
            } catch (Exception $e) {
                return $translate->_($e->getMessage());
            }
                    
            $editingPlugin->updateGeometryForEdit($id, $row['table'], $row['geometry'], array(
                'primary' => 'hgid'
            ));
        }
                        
        $this->deleteDirectory(GT_TMP_PATH . DIR_SEP . $fileName);
        unlink(GT_TMP_PATH . DIR_SEP . $fileName . '.zip');
                        
        //Update history extent
        $mainHistory->ediItem($historyId, array(
            'bounds' => $this->_bounds
        ));
                        
        return $translate->_('fileUploaded');
    }

    public function upload ($fileName, array $options = array())
    {
        
        $layers = new Giscuit_Db_Table_Layers();
        
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        
        if ($tables == false) {
            throw new Giscuit_Exception("Package does not change anything");

        }
        
        //Clear
        $this->_hisotryTables = $this->_changes = $this->_errors = $filesToExtract = $this->_bounds = array();
        $this->_changed = false;
        
        //Get current user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        //Set history id
        $historyId = explode('_', $fileName);
        $historyId = intval($historyId[count($historyId) - 1]);
        
        if (empty($historyId)) {
            throw new Giscuit_Exception("Package does not exist");

        }
        
        $mainHistory = new Giscuit_Db_Table_History();
        $result = $mainHistory->getItem($historyId, array(
            'whereFetchUserId' => $userId , 
            'withGidlist' => true
        ));
        
        if (empty($result)) {
            throw new Giscuit_Exception("Package does not exist");
        }
        
        if ($result->active == 't') {
            throw new Giscuit_Exception("Package already activated");
        }
        
        //Create zip archive
        $zip = new ZipArchive();
        $res = $zip->open(GT_TMP_PATH . DIR_SEP . $fileName . '.zip');
        if (! $res) {
            throw new Giscuit_Exception("Cannot open archive for writing");
        }
        
        //Get valid file names
        for ($i = 0; $i < $zip->numFiles; $i ++) {
            $archivePath = $zip->getNameIndex($i);
            $archiveParts = pathinfo($archivePath);
            
            if (isset($archiveParts['extension']) && in_array(strtolower($archiveParts['extension']), $this->_desc[$result->ext]['extensions'])) {
                $filesToExtract[] = $archivePath;
            }
        }
        
        if (empty($filesToExtract)) {
            throw new Giscuit_Exception('Package does not change anything');

        }
        
        //Create working directory
        if (! file_exists(GT_TMP_PATH . DIR_SEP . $fileName)) {
            mkdir(GT_TMP_PATH . DIR_SEP . $fileName);
        }
        
        //Extract
        $zip->extractTo(GT_TMP_PATH, $filesToExtract);
        
        //Move files to working directory
        foreach ($filesToExtract as $file) {
            $fileParts = pathinfo($file);
            rename(GT_TMP_PATH . DIR_SEP . $file, GT_TMP_PATH . DIR_SEP . $fileName . DIR_SEP . $fileParts['basename']);
        }
        
        $zip->close();
        
        //Path
        $path = GT_TMP_PATH . DIR_SEP . $fileName;
        
        //Gidlist
        $gidlist = $result->gidlist;
        
        foreach ($tables as $table) {
            $settings = $table->getSettings();

            //Create form
            $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $settings['class'] . '.xml';
            $configContent = new Zend_Config_Xml($configPath);
            $formName = 'Giscuit_Form_Layer_' . $settings['class'];
            $form = new $formName($configContent);

            $historyTableName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
            $historyTable = new $historyTableName(array(
                'name' => $settings['table'] ,
                'withHistory' => true ,
            ));
            $historyTable->setSettings($settings);
            
            $this->_hisotryTables[] = $historyTable;
            
            $validateOptions = array (
                'gidlist' => $gidlist[$settings['table']] , 
                'settings' => $settings , 
                'historyId' => $historyId , 
                'path' => $path , 
                'date' => $result->fetch_date , 
                'ext' => $result->ext
            );
            if (isset($options['skip'])) {
                $validateOptions['skip'] = $options['skip'];
            }
            if (isset($options['overwrite'])) {
                $validateOptions['overwrite'] = $options['overwrite'];
            }
            
            $this->validate($table, $form, $historyTable, $validateOptions);
        }
        
        if (! $this->_changed) {
            throw new Giscuit_Exception('Package does not change anything');

        }
        
        return $historyId;
    
    }

    public function validate ($table, $form, $history, array $options = array())
    {
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $info = $table->info();
        $settings = $table->getSettings();
        
        $packageDate = floatval($options['date']);
        
        $tableName = $info['name'];
        $layerName = $settings['name'];
        
        $pkeyName = $table->getPrimaryKey(array(
            'name' => true
        ));
        
        //
        if (! file_exists($options['path'] . DIR_SEP . $tableName . '.' . $options['ext'])) {
            $options['ext'] = strtoupper($options['ext']);
        }
        
        $layerClassName = 'Giscuit_Map_Layer_' . $settings['class'];
        
        if (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Line')) {
            $geometryType = 'line';
        } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Polygon')) {
            $geometryType = 'polygon';
        } elseif (is_subclass_of($layerClassName, 'Giscuit_Map_Layer_Point')) {
            $geometryType = 'point';
        } else {
            throw new Giscuit_Exception('Invalid layer class');
        }
        
        $msVersion = ms_GetVersionInt();
        
        //
        $reservedFields = array(
            'hgid' ,
            'h_id' ,
            'hstatus' ,
            'changed_date' ,
            'changed_date_timestamp' ,
            'created_by' ,
            'verified_by'
        );
        
        //Get records
        $recordsGidList = $records = array();
        
        $layer = new $layerClassName($this->_map, array(
            'class' => $settings['class']
        ));
        $layer->setConnectionType(MS_OGR);
        $layer->set('connection', $options['path'] . DIR_SEP . $tableName . '.' . $options['ext']);
        $status = @$layer->open();
        if ($status == MS_FAILURE) {
            return;
        }
        $status = $layer->whichShapes($this->_map->extent);
        while ($shape = $layer->nextShape()) {
            $values = array_change_key_case($shape->values);
            foreach ($values as $key => $value) {
                if($value === '') {
                    $values[$key] = NULL;    
                }
            }
            
            $records[$values[$pkeyName]] = $values;
            $shapes[$values[$pkeyName]] = $shape;
                
            $values[$pkeyName] = intval($values[$pkeyName]);
            $recordsGidList[] = $values[$pkeyName];
        }
        $layerExtent = $layer->getExtent();
        $layer->close();
        
        //Get db records
        $dbSelect = $history->getSelect('all_table', array(
            'whereStm' => "\"$tableName\".\"$pkeyName\" IN (" . implode(',', $recordsGidList) . ')' ,
            'fields' => array(
                'changed_date_timestamp' => new Zend_Db_Expr("extract(epoch from globhist.activate_date)")
            )
        ));
        
        $layer->setPostgisOptions();
        $layer->setData($dbSelect, 'hgid');
        $status = @$layer->open();
        if ($status == MS_FAILURE) {
            return;
        }
        $status = $layer->whichShapes($this->_map->extent);
        while ($shape = $layer->nextShape()) {
            $values = array_change_key_case($shape->values);
            foreach ($values as $key => $value) {
                if($value === '') {
                    $values[$key] = NULL;    
                }
            }
            
            $dbRecords[$values[$pkeyName]] = $values;
            $dbShapes[$values[$pkeyName]] = $shape;
        }
        $layer->close();
           
        //Add geometry validators
        $theGeomElement = $form->createElement('text', 'the_geom');
        $theGeomElement->addValidator('Geometry');
                
        //Float
        $floatTypes = array(
            'float4' , 
            'float8' , 
            'decimal' , 
            'numeric'
        );
        $floatFields = array();
        foreach ($info["metadata"] as $elementKey => $element) {
            if(in_array($element['DATA_TYPE'], $floatTypes)) {
                if($form->getElement($elementKey)) {
                    $floatFields[] = $elementKey;
                }
            }
        }
        
        //Date
        $dateFields = array();
        foreach($form->getElements() as $element) {
            if($element instanceof Giscuit_Form_Element_Date) {
                $dateFields[] = $elementName = $element->getName();        
            }
        }
        
        $index = 0;
        $changedGisList = array();
        $resolvedElements = array();
        if (! isset($options['skip'])) {
            $options['skip'] = array();
        }
        if (! isset($options['overwrite'])) {
            $options['overwrite'] = array();
        }
        $resolvedElements = array_merge($resolvedElements, $options['skip'], $options['overwrite']);

        foreach ($records as $recordKey => $record) {
            $index ++;

            //FIXME:
            $record = array_map('trim', $record);
            
            $form->reset();
            //TODO: Temporary
            $form->addElement(clone $theGeomElement);     
            
            //Validate file upload
            foreach($form->getElements() as $element) {
                if($element instanceof Zend_Form_Element_File) {
                    $elementName = $element->getName();
                    $form->removeElement($elementName);
                    
                    $element = $form->createElement('text', $elementName);
                    $element->addValidator('Int');
                    $form->addElement($element);
                }
            }
                        
            $form->populate($record);
            
            if ($form->isValid(array_merge($record, array('the_geom' => $shapes[$recordKey]->toWKT())))) {
                //Get validated record
                $validRecord = $form->getValues();
                                
                foreach ($validRecord as $key => $value) {
                    if($value === '') {
                        $validRecord[$key] = NULL;    
                    }
                }
                
                //Update
                if (! empty($record[$pkeyName])) {
                    //Add to returned gid list
                    $changedGisList[] = $record[$pkeyName];
                    
                    //Set gid for valid record
                    $validRecord[$pkeyName] = $record[$pkeyName];
                                        
                    if(! isset($dbRecords[$recordKey])) {
                        if (! in_array($record[$pkeyName], $resolvedElements)) {
                            $form->getElement('the_geom')->addError($translate->_('deleteConflict'));
                        } else {
                            if (in_array($record[$pkeyName], $options['skip'])) {
                                //Do nothing
                            } else if (in_array($record[$pkeyName], $options['overwrite'])) {
                                unset($validRecord['the_geom']);
                                
                                $validRecord['hstatus'] = 'U';
                                $validRecord['h_id'] = $options['historyId'];

                                $this->_changes[] = $history->addItemForHistory($validRecord, array(
                                     'geometry' => $shapes[$recordKey]->toWKT()
                                ));

                                $this->_changed = true;
                            }
                        }
                    } else {
                        $dbRecord = $dbRecords[$recordKey]; 
                        
                        $recordDate = floatval($dbRecord['changed_date_timestamp']);

                        if ($recordDate > $packageDate && ! in_array($record[$pkeyName], $resolvedElements)) {
                            $form->getElement('the_geom')->addError($translate->_('updateConflict'));
                        } else if (! in_array($record[$pkeyName], $options['skip'])) {
                            //Date
                            foreach ($dateFields as $element) {
                                if(strtotime($dbRecord[$element]) == strtotime($validRecord[$element])) {
                                    $dbRecord[$element] = $validRecord[$element];    
                                }    
                            }
                                                     
                            //Float
                            foreach ($floatFields as $element) {
                                if(number_format($dbRecord[$element], 15, '.' ,'') == 
                                    number_format($validRecord[$element], 15, '.' ,'')) {

                                    $dbRecord[$element] = $validRecord[$element]; 
                                }    
                            }
                            
                            //
                            foreach ($reservedFields as $value) {
                                unset($dbRecord[$value]);
                            }
                            unset($validRecord['the_geom']);
                            
                            //Check if record has changed
                            if (array_diff_assoc($validRecord, $dbRecord) || 
                                ! $shapes[$recordKey]->equals($dbShapes[$recordKey])) {
                                    
                                $validRecord['hstatus'] = 'U';
                                $validRecord['h_id'] = $options['historyId'];

                                $this->_changes[] = $history->addItemForHistory($validRecord, array(
                                     'geometry' => $shapes[$recordKey]->toWKT()
                                ));

                                $this->_changed = true;
                            } else {

                            }
                        
                        }
                    
                    }
                    
                //Add
                } else {
                    $validRecord['hstatus'] = 'C';
                    $validRecord['h_id'] = $options['historyId'];
                    unset($validRecord['the_geom']);
                    
                    $this->_changes[] = $history->addItemForHistory($validRecord, array(
                        'geometry' => $shapes[$recordKey]->toWKT()
                    ));
                    $this->_changed = true;
                }
            
            }
            
            $messages = $form->getMessages();
            
            if (! empty($messages)) {
                $errors = array(
                    'index' => $index , 
                    'name' => $layerName , 
                    'gid' => $record[$pkeyName] , 
                    'geometry' => $form->getValue('the_geom'),
                    'messages' => $form->getMessages()
                );
                
                $this->_errors[$tableName][] = $errors;
                $this->_changed = true;
            }
        
            $form->removeElement('the_geom');
        }
         
        //Delete
        $deleteList = array_diff($options['gidlist'], $changedGisList);
        
        if (! empty($deleteList)) {
            $this->_changed = true;
            
            //Get db records
            $dbDeleteSelect = $history->getSelect('all_table', array(
                'geometryAsWkt' => true,
                'whereStm' => "\"$tableName\".\"$pkeyName\" IN (" . implode(',', $deleteList) . ')' ,
                'fields' => array(
                    'changed_date_timestamp' => new Zend_Db_Expr("extract(epoch from globhist.activate_date)")
                )
            ));
            
            $deleteRecords = $history->fetchAll($dbDeleteSelect)->toArray();
            
            foreach ($deleteRecords as $record) {
                $recordDate = floatval($record['changed_date_timestamp']);

                if($recordDate > $packageDate && ! in_array($record[$pkeyName], $options['overwrite'])) {
                    $errors = array(
                        'index' => NULL , 
                        'name' => $layerName , 
                        'gid' =>  $record[$pkeyName], 
                        'geometry' => $record['the_geom'],
                        'messages' => array(
                            'the_geom' => array(
                                $translate->_('updateConflict')
                            )
                        )
                    );
                
                    $this->_errors[$tableName][] = $errors;
                } else {
                    foreach ($record as $key => $value) {
                        if(in_array($key, $reservedFields)) {
                            unset($record[$key]);
                        }
                    }
                    
                    $geometry = $record['the_geom'];
                    unset($record['the_geom']);
 
                    $record['hstatus'] = 'D';
                    $record['h_id'] = $options['historyId'];

                    $this->_changes[] = $history->addItemForHistory($record, array(
                        'geometry' => $geometry
                    ));
                    
                }
            }
        }

        if($this->_changed == true) {
            $this->_mergeBounds($layerExtent);
        }

    }

    public function _mergeBounds ($bounds)
    {
        
        //MinX
        if (! isset($this->_bounds['minx']) || $bounds->minx < $this->_bounds['minx']) {
            $this->_bounds['minx'] = $bounds->minx;
        }
        
        //MinY
        if (! isset($this->_bounds['miny']) || $bounds->miny < $this->_bounds['miny']) {
            $this->_bounds['miny'] = $bounds->miny;
        }
        
        //MaxX
        if (! isset($this->_bounds['maxx']) || $bounds->maxx > $this->_bounds['maxx']) {
            $this->_bounds['maxx'] = $bounds->maxx;
        }
        
        //MaxY
        if (! isset($this->_bounds['maxy']) || $bounds->maxy > $this->_bounds['maxy']) {
            $this->_bounds['maxy'] = $bounds->maxy;
        }
    }

    public function activateAction ()
    {
        /*
        $settings = Zend_Registry::get('settings');
        
        if (! isset($settings->compareMode)) {
            $settings->compareMode = 'f';
        }
        
        $this->view->compareMode = $settings->compareMode;
        */
    }

    public function activategetAction ()
    {
        $result['replyCode'] = 200;
        $result['records'] = array();
        
        $history = new Giscuit_Db_Table_History();
        
        //Get editable layer names
        $editTableNames = array();
        $layers = new Giscuit_Db_Table_Layers();

        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        foreach($tables as $table) {
            $editTableNames[] = $table->info('name');
        }
               
        //Check if the user can activate the layers in the package
        $records = $history->getInactiveItems()->toArray();
        
        foreach ($records as $key => $row) {
            
            if (! is_array($row['gidlist'])) {
                break;
            }
            
            $layers = array_keys($row['gidlist']);
            
            foreach ($layers as $layer) {
                if (! in_array($layer, $editTableNames)) {
                    unset($records[$key]);
                }
            }
            unset($records[$key]['gidlist']);
        }
        
        if (! empty($records)) {
            $result['records'] = array_values($records);
        }
        
        $this->_helper->json($result);
    }

    public function activatesubmitAction ()
    {
        $result['replyCode'] = 200;
        
        $historyId = intval($this->getRequest()->getUserParam('id', 0));
        
        $historyTable = new Giscuit_Db_Table_History();
        if ($historyTable->isAllowed($historyId) == true) {
            
            $layers = new Giscuit_Db_Table_Layers();
        
            $layers = new Giscuit_Db_Table_Layers();
            $tables = $layers->getEditTables(array(
                'whereHistory' => true
            ));
            
            foreach ($tables as $table) {
                $settings = $table->getSettings();
                
                $historyName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
                $history = new $historyName(array(
                    'name' => $settings['table'] ,
                    'withHistory' => true
                ));
                $history->setSettings($settings);
            
                $history->activateHistory($historyId, $table);
            }
        
        }
        
        $this->_helper->json($result);
    }

    public function activatedeleteAction ()
    {
        $result['replyCode'] = 200;
        
        $historyId = intval($this->getRequest()->getParam('id', 0));
        
        $historyTable = new Giscuit_Db_Table_History();
        if ($historyTable->isAllowed($historyId) == true) {
            $where = $historyTable->getAdapter()->quoteInto('id = ?', $historyId);
            $historyTable->delete($where);
        }
        
        $this->_helper->json($result);
    }

    public function downloadgetAction ()
    {
        $result['replyCode'] = 200;
        $result['form'] = $this->getDownloadForm()->render();
        
        $this->_helper->json($result);
    }

    public function downloadcreateAction ()
    {
        $result['replyCode'] = 200;
        
        $config = Zend_Registry::get('config');
        $translate = Zend_Registry::get('Zend_Translate');
        
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $request = $this->getRequest();
        $gidlist = array();
        
        $connectionString = Giscuit_Controller_Plugin_Init::getDbConnectionString();
        
        $form = $this->getDownloadForm();
        $form->populate($request->getParams());
        
        if (! $form->isValid($request->getParams())) {
            $result['form'] = $form->render();
        } else {
            
            $ext = $form->getValue('ext');
            
            $layers = new Giscuit_Db_Table_Layers();
            $tables = $layers->getEditTables(array(
                'whereHistory' => true
            ));
       
            if (empty($tables)) {
                $result['msg'] = $translate->_('nothingForDownload');
            } else {
                
                //Validate geometry
                $geometryValidator = new Giscuit_Validate_Geometry();
                $geometry = $request->getParam('geometry');
                if (! $geometryValidator->isValid($geometry)) {
                    throw new Giscuit_Exception('Invalid geometry');

                }
                $theGeom = "ST_GeometryFromText('$geometry', {$config->map->srid})";
                
                $history = new Giscuit_Db_Table_History();
                $hId = $history->addItem($form);
                
                $date = new Zend_Date();
                $dirName = $hId . '_' . md5(uniqid(rand(), true));
                $dirPath = GT_TMP_PATH . DIR_SEP . $dirName;
                
                //Create folder
                mkdir($dirPath, 0755);
                
                //Create zip archive
                $zip = new ZipArchive();
                $delList = array();
                
                $res = $zip->open($dirPath . '.zip', ZipArchive::CREATE);
                if (! $res) {
                    throw new Giscuit_Exception("Cannot open archive for writing");
                }
                
                foreach ($tables as $table) {
                    $info = $table->info();
                    
                    //Generate shape file
                    $select = $table->getSelect('all_table', array(
                        'whereIntersection' => $theGeom ,
                        'limit' => $this->limit
                    ));
                    
                    //Quote
                    $select = str_replace('"', '\"', (string) $select);
                    
                    $cmd = "-a_srs EPSG:{$config->map->srid} -f \"{$this->_desc[$ext]['name']}\" \"" . $dirPath . DIR_SEP . $info['name'] . ".{$ext}\"" 
                        . " PG:\"{$connectionString}\""
                        . " -fieldTypeToString Date,Time,DateTime" 
                        . " -sql \"$select\"";
                    
                    Giscuit_Exec::exec('ogr2ogr', $cmd, GT_GDAL_BIN_PATH);
                    
                    foreach ($this->_desc[$ext]['extensions'] as $value) {
                        $fileName = $info['name'] . '.' . $value;
                        if (file_exists($dirPath . DIR_SEP . $fileName)) {
                            $zip->addFile($dirPath . DIR_SEP . $fileName, $fileName);
                            $delList[] = $dirPath . DIR_SEP . $fileName;
                        }
                    }
                    
                    //Get gid list
                    $gidlist[$info['name']] = array();
                    $pkeyName = $table->getPrimaryKey(array(
                        'name' => true
                    ));
                    $rows = $table->getItems(array(
                        'whereIntersection' => $theGeom
                    ));
                    foreach ($rows as $row) {
                        $gidlist[$info['name']][] = $row[$pkeyName];
                    }
                
                }
                
                if ($zip->numFiles == 0) {
                    $result['msg'] = $translate->_('nothingForDownload');
                } else {
                    $history->setGidlist($hId, $gidlist);
                    
                    $result['file'] = $dirName;
                    $result['msg'] = $translate->_('fileCreated');
                }
                
                $zip->close();
                
                foreach ($delList as $value) {
                    unlink($value);
                }
                
                rmdir($dirPath);
            
            }
        
        }
        
        $this->_helper->json($result);
    }

    public function downloadAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $file = $this->getRequest()->getParam('file');
        $parts = explode('_', $file);
        
        if (empty($parts[1]) || ! preg_match('/^[a-f0-9]{32}$/', $parts[1])) {
            return;
        }
        
        $fullPath = GT_TMP_PATH . DIR_SEP . $file . '.zip';
        if ($fd = fopen($fullPath, "r")) {
            header("Content-type: application/octet-stream");
            header("Content-Disposition: filename=\"giscuit_package_{$parts[0]}.zip\"");
            header("Content-length: " . filesize($fullPath));
            header("Cache-control: private");
            while (! feof($fd)) {
                $buffer = fread($fd, 2048);
                echo $buffer;
            }
        }
        fclose($fd);
    }
    
    public function reportbackAction ()
    {
        $result['replyCode'] = 200;
        
        Zend_Registry::get('settings')->historyId = NULL;
        
        $this->_helper->json($result);
    }

    public function reportAction ()
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $historyId = intval($request->getParam('id', 0));
        
        if (empty($historyId)) {
            throw new Giscuit_Exception("Package does not exist");
        }
        
        Zend_Registry::get('settings')->historyId = $historyId;
        
        $result = array();
        
        $layers = new Giscuit_Db_Table_Layers();
        $tables = $layers->getEditTables(array(
            'whereHistory' => true
        ));
        
        foreach ($tables as $table) {
            $settings = $table->getSettings();
            
            $historyName = 'Giscuit_Db_Table_Layer_' . $settings['class'];
            $history = new $historyName(array(
                'name' => $settings['table'] ,
                'withHistory' => true
            ));
            $history->setSettings($settings);
            
            $report = $this->getTableReport($historyId, $table, $history);
            if($report != false) {
                $result[$settings['table']] = $report; 
            } 
        }
        
        $this->_helper->json($result);
    
    }

    public function getTableReport ($hId, $table, $history)
    {
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $translate = Zend_Registry::get('Zend_Translate');
        $settings = $table->getSettings();
        
        $pkeyName = $table->getPrimaryKey(array(
            'name' => true
        ));
                        
        $records = array();
        
        $unsedFields = array(
            'hgid' ,
            'h_id' ,
            //'hstatus' ,
             'changed_date' ,
             'created_by' ,
            'verified_by'
        );
        
        //New values
        $newSelect = $history->getSelect('all', array(
            'whereHistoryId' => $hId ,
            'geometryAsWkt' => true , 
            'fields' => array(
                'hstatus',
                'gid'
            )
        ));
        
        $newRows = $history->fetchAll($newSelect)->toArray();
        
        if(empty($newRows)) {
            return false;
        }
        
        $newGids = array();
        foreach ($newRows as $key => $row) {
            foreach ($unsedFields as $unsedField) {
                unset($newRows[$key][$unsedField]);
            }
            if (! empty($row['gid'])) {
                $newGids[] = $row['gid'];
            }
        }
        
        if(!empty($newGids)) {
        
            //Old values
            $oldTableName = $table->info('name');
            $whereStm = "$oldTableName.$pkeyName = " . implode(" OR $oldTableName.$pkeyName = ", $newGids);
            
            $oldSelect = $table->getSelect('all', array(
                'geometryAsWkt' => true , 
                'whereStm' => $whereStm
            ));
            
            $oldRows = $history->fetchAll($oldSelect)->toArray();
            
            //Add update values
            foreach ($oldRows as $oldKey => $oldValue) {
                foreach ($newRows as $newKey => $newValue) {
                    if ($oldValue[$pkeyName] == $newValue[$pkeyName] && $newValue['hstatus'] == 'U') {
                        $diff = array_diff($newValue, $oldValue);
                        
                        foreach ($diff as $key => $value) {
                            //Status
                            if ($key == 'hstatus') {
                                $oldRows[$oldKey][$key] = $value;
                                continue;
                            }
                            
                            $oldRows[$oldKey][$key] = array(
                                $oldRows[$oldKey][$key] , 
                                $value
                            );
                        
                        }
                        
                        $records[] = $oldRows[$oldKey];
                    }
                }
            }
        
        }
        
        //Add create and delete values
        foreach ($newRows as $newKey => $newValue) {
            if ($newValue['hstatus'] != 'U') {
                $records[] = $newValue;
            }
        }
                
        $fields = array_keys($records[0]);
        
        //Column definitions
        $columnDefs = array(
            array(
                'key' => 'hstatus' , 
                'label' => $translate->_('hstatus') , 
                'sortable' => true , 
                'formatter' => "formatStatusField" , 
                'resizeable' => true
            )
        );
        
        foreach ($fields as $field) {
            if ($field == 'hstatus' || $field == 'the_geom') {
                continue;
            }
            
            $arrObj = new ArrayObject(array(
                'key' => $field , 
                'label' => $translate->_($field) , 
                'sortable' => true , 
                'resizeable' => true
            ));
            
            $columnDefs[] = $arrObj;
        }
        
        $result = array(
            'records' => $records , 
            'fields' => $fields , 
            'name' => $settings['name'] , 
            'columns' => $columnDefs
        );
        
        return $result;
    
    }
    
    public function getDownloadForm ()
    {
        $form = new Giscuit_Form_HistorydataDownload();
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtHistorydataDownloadForm'
        ));
        return $form;
    }

    public function getUploadForm ()
    {
        $form = new Giscuit_Form_HistorydataUpload();
        
        $form->setMethod('post');
        $form->addAttribs(array(
            'id' => 'gtHistorydataUploadForm'
        ));

        return $form;
    }

    public static function deleteDirectory ($dir)
    {
        if (! file_exists($dir))
            return true;
        if (! is_dir($dir))
            return unlink($dir);
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..')
                continue;
            if (! Tool_HistorydataController::deleteDirectory($dir . DIRECTORY_SEPARATOR . $item))
                return false;
        }
        return rmdir($dir);
    }

}
