<?php

class Tool_RoutingController extends Zend_Controller_Action
{

    public function indexAction()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $result['replyCode'] = 200;

        $routingCostColumnsTable = new Giscuit_Db_Table_RoutingCostColumns();
        
        $records = $this->_getRoutableRecords();
        
        $routableLayers = array();
        foreach ($records as $record) {
            $layer = array();
            $layer['id'] = $record['routing_id'];
            $layer['name'] = $record['name'];
            $layer['source'] = $record['routing_source'];
            $layer['reverse_source'] = $record['routing_reverse_source'];
            
            //
            $algorithms = explode(',', $record['routing_algorithm']);
            foreach ($algorithms as $algorithm) {
                $layer['algorithms'][$algorithm] = $translate->_(Giscuit_Db_Table_LayerRouting::$algorithmNames[$algorithm]);
            }

            //
            $costs = $routingCostColumnsTable->getItems(array(
                'whereLayerId' => $layer['id'] ,
                'withCurrentLanguage' => true
            ));
            foreach ($costs as $cost) {
                $layer['costs'][$cost->id] = $cost->name;    
            }
            
            $routableLayers[] = $layer;
        }
        
        $result['layers'] = $routableLayers;
        
        $this->_helper->json($result);
    }
    
    public function getdirectionsAction () 
    {
        $result['replyCode'] = 200;
        
        $request = $this->getRequest();
        $layerId = intval($request->getParam('layer'));
        $costId = intval($request->getParam('cost'));
        
        $algorithmId = intval($request->getParam('algorithm'));
        $points = Zend_Json::decode($request->getParam('points'));

        for ($i = 0; $i < count($points); $i++) {
            $points[$i][0] = floatval($points[$i][0]);
            $points[$i][1] = floatval($points[$i][1]);
        }
                
        //Get layer
        $layerSettings = $this->_getRoutableRecords(array(
            'whereIds' => array($layerId),
            'withRoutingCostCoefColumn' => $costId
        ));
        if (! count($layerSettings)) {
            throw new Giscuit_Db_Table_Exception("Invalid layer id ({$layerId})");
        }
        
        $layerSettings = $layerSettings[0];
        $layerTableName = 'Giscuit_Db_Table_Layer_' . $layerSettings['class'];
        $layerTable = new $layerTableName(array(
            'name' => $layerSettings['table']
        ));
        $layerTable->setSettings($layerSettings);
                
        //Get edges
        $edges = array();
        foreach ($points as $point) {
            $edges[] = Giscuit_Db_Table_LayerRouting::nearestEdge($layerTable, $point[0], $point[1]);
        }
        
        //TSP
        if($algorithmId == 4) {
            if(count($edges) > 2) {
                $edges = Giscuit_Db_Table_LayerRouting::orderEdgesByTsp($layerTable, $edges);
            }    
            $algorithmId = 1;
        }
        
        if($algorithmId == 5) {
            if(count($edges) > 2) {
                $edges = Giscuit_Db_Table_LayerRouting::orderEdgesByTsp($layerTable, $edges);
            }
            $algorithmId = 2;
        }
        
        //Get paths
        $directed = $layerSettings['routing_directed'] == 't' ? true : false;
        $reverse_cost = $layerSettings['routing_reverse_cost'] == 't' ? true : false;
        $cost_coef_column = @$layerSettings['cost_coef_column'];
        $reverse_cost_coef_column = @$layerSettings['reverse_cost_coef_column'];
        
        $paths = array();
        for ($i = 0; $i < count($edges) - 1; $i++) {
            $path = Giscuit_Db_Table_LayerRouting::getPath(
                $layerTable, $edges[$i], $edges[$i+1], $algorithmId, 
                $directed, $reverse_cost, 
                $cost_coef_column, $reverse_cost_coef_column
            );

            if(isset($path['path'])) {
                array_push($paths, $path);
            }
        }
        
        $result['paths'] = $paths;

        $this->_helper->json($result);
    }
    
    protected function _getRoutableRecords (array $options = array())
    {
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $layersTable = new Giscuit_Db_Table_Layers();
        
        if(! empty($userId)) {
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'whereActive' => 't' , 
                'withAcl' => true,
                'withRouting' => true,
                'whereRoutingActive' => 't',
                'withLayerSettings' => true ,
            );
        } else {
            $mapOptions = array(
                'orderDirection' => 'asc' ,
                'wherePublish' => 't' , 
                'whereActive' => 't' , 
                'withPublish' => true ,
                'withRouting' => true,
                'whereRoutingActive' => 't',
            );
        }
        
        $mapOptions = array_merge($mapOptions, $options);
            
        $result = array_values($layersTable->getItems($mapOptions));
        
        return $result;
    }
    
}