--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET standard_conforming_strings = off;
SET check_function_bodies = false;
SET client_min_messages = warning;
SET escape_string_warning = off;


--
-- Name: first_agg(anyelement, anyelement); Type: FUNCTION;
--

CREATE FUNCTION first_agg(anyelement, anyelement) RETURNS anyelement
    AS $_$
        SELECT CASE WHEN $1 IS NULL THEN $2 ELSE $1 END;
$_$
    LANGUAGE sql STABLE;


--
-- Name: layers_tree(); Type: FUNCTION;
--

CREATE FUNCTION layers_tree() RETURNS trigger
    AS $$
DECLARE 
    parent_path TEXT DEFAULT '';
    table_name text := TG_TABLE_SCHEMA || '.' || TG_TABLE_NAME;
BEGIN

    IF NEW.parent_id IS NOT NULL THEN
        EXECUTE 'SELECT path FROM ' || table_name || ' WHERE id = ' || NEW.parent_id INTO parent_path;
        NEW.path := parent_path||NEW.id||'/';
    END IF;
    
    IF NEW.parent_id IS NULL THEN
        NEW.path := '/'||NEW.id||'/';
    END IF;

    return NEW;
END;
$$
    LANGUAGE plpgsql;


--
-- Name: lo_delete(); Type: FUNCTION;
--

CREATE FUNCTION lo_delete() RETURNS trigger
    AS $$BEGIN
    perform lo_unlink(OLD.oid);
    RETURN OLD;
END;$$
    LANGUAGE plpgsql;


--
-- Name: first(anyelement); Type: AGGREGATE;
--

CREATE AGGREGATE first(anyelement) (
    SFUNC = first_agg,
    STYPE = anyelement
);

  
SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: base; Type: TABLE; 
--

CREATE TABLE base (
    id integer NOT NULL,
    name_id integer,
    selected character(1) DEFAULT 'f'::bpchar,
    format character varying(10),
    "order" integer,
    active character(1) DEFAULT 'f'::bpchar,
    minscaledenom integer,
    maxscaledenom integer,
    parent_id integer
);


--
-- Name: base_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE base_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: base_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE base_id_seq OWNED BY base.id;


--
-- Name: classes; Type: TABLE; 
--

CREATE TABLE classes (
    id integer NOT NULL,
    layer_id integer,
    name character varying(100),
    expression pg_catalog.text,
    minscaledenom integer,
    maxscaledenom integer,
    "order" integer,
    label_size integer DEFAULT 10,
    label_color character varying(7) DEFAULT '#000000'::character varying,
    label_font character varying(100),
    label_active character(1) DEFAULT 'f'::bpchar,
    label_buffer integer DEFAULT 0,
    label_minfeaturesize integer,
    label_backgroundcolor character varying(7),
    label_angle character varying(10) DEFAULT 'auto'::character varying,
    label_position integer DEFAULT 110,
    label_repeatdistance integer,
    label_align integer DEFAULT 1,
    label_maxlength integer DEFAULT 0,
    label_priority integer DEFAULT 1,
    label_wrap character varying(1),
    legend character(1) DEFAULT 't'::bpchar
);


--
-- Name: classes_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE classes_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: classes_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE classes_id_seq OWNED BY classes.id;


--
-- Name: comm_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE comm_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: comm; Type: TABLE; 
--

CREATE TABLE comm (
    id integer DEFAULT nextval('comm_id_seq'::regclass) NOT NULL,
    text pg_catalog.text,
    part integer,
    user_id integer,
    date timestamp with time zone
);


--
-- Name: fields; Type: TABLE; 
--

CREATE TABLE fields (
    id integer NOT NULL,
    layer_id integer,
    name character varying(100),
    expression pg_catalog.text,
    visibility smallint DEFAULT 1,
    escape character(1) DEFAULT 't'::bpchar,
    "order" integer
);


--
-- Name: fields_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE fields_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: fields_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE fields_id_seq OWNED BY fields.id;


--
-- Name: joins; Type: TABLE; 
--

CREATE TABLE joins (
    id integer NOT NULL,
    expression pg_catalog.text,
    "table" character varying(100),
    layer_id integer,
    "order" integer
);


--
-- Name: joins_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE joins_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: joins_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE joins_id_seq OWNED BY joins.id;


--
-- Name: files; Type: TABLE; 
--

CREATE TABLE files (
    id integer NOT NULL,
    oid oid,
    by_user_id integer,
    date timestamp with time zone,
    name character varying(255),
    for_layer_id integer
);


--
-- Name: files_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE files_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: files_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE files_id_seq OWNED BY files.id;


--
-- Name: history; Type: TABLE; 
--

CREATE TABLE history (
    id bigint NOT NULL,
    fetch_user_id integer,
    fetch_date timestamp with time zone,
    active character(1) DEFAULT 'f'::bpchar,
    comment pg_catalog.text,
    upload_extent public.geometry,
    gidlist pg_catalog.text,
    activate_user_id integer,
    upload_date timestamp with time zone,
    activate_date timestamp with time zone,
    ext character varying(3),
    CONSTRAINT enforce_dims_extent CHECK ((public.ST_NDims(upload_extent) = 2))
);


--
-- Name: history_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE history_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: history_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE history_id_seq OWNED BY history.id;


--
-- Name: layer_defaults; Type: TABLE; 
--

CREATE TABLE layer_defaults (
    id integer NOT NULL,
    label_size integer DEFAULT 10,
    label_color character varying(7) DEFAULT '#000000'::character varying,
    label_fields pg_catalog.text,
    view character(1) DEFAULT 'f'::bpchar,
    "select" character(1) DEFAULT 'f'::bpchar,
    label character(1) DEFAULT 'f'::bpchar,
    edit character(1) DEFAULT 'f'::bpchar,
    "order" integer
);


--
-- Name: layer_owsclient; Type: TABLE; 
--

CREATE TABLE layer_owsclient (
    id integer NOT NULL,
    owsclient_uri character varying(200),
    owsclient_image_format character varying(30),
    owsclient_version character varying,
    owsclient_layers character varying(200),
    owsclient_info_format character varying(30),
    owsclient_auth_username character varying(100),
    owsclient_auth_password character varying(200),
    owsclient_auth_type character varying(10),
    owsclient_update_interval integer,
    owsclient_update_date timestamp with time zone
);


--
-- Name: layer_publish; Type: TABLE; 
--

CREATE TABLE layer_publish (
    wms_private character(1) DEFAULT 'f'::bpchar,
    wms_public character(1) DEFAULT 'f'::bpchar,
    wfs_private character(1) DEFAULT 'f'::bpchar,
    wfs_public character(1) DEFAULT 'f'::bpchar,
    wcs_private character(1) DEFAULT 'f'::bpchar,
    wcs_public character(1) DEFAULT 'f'::bpchar,
    wcs_formats pg_catalog.text,
    publish character(1) DEFAULT 'f'::bpchar,
    format character varying(10),
    id integer NOT NULL,
    selected character(1) DEFAULT 'f'::bpchar,
    destination character varying(50)
);


--
-- Name: layer_settings; Type: TABLE; 
--

CREATE TABLE layer_settings (
    id integer NOT NULL,
    layer_id integer,
    user_id integer,
    view character(1) NOT NULL,
    "select" character(1) NOT NULL,
    label character(1) NOT NULL,
    label_size smallint,
    label_color character varying(7),
    label_fields pg_catalog.text,
    "order" integer,
    edit character(1) NOT NULL
);


--
-- Name: layer_settings_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE layer_settings_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: layer_settings_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE layer_settings_id_seq OWNED BY layer_settings.id;

--
-- Name: layer_settings_unique; Type: CONSTRAINT; 
-

ALTER TABLE ONLY layer_settings
    ADD CONSTRAINT layer_settings_unique UNIQUE (layer_id, user_id);

    
--
-- Name: layers; Type: TABLE; 
--

CREATE TABLE layer_permissions (
    id integer NOT NULL,
    user_id integer,
    view_ids pg_catalog.text,
    editgeom_ids pg_catalog.text,
    editattr_ids pg_catalog.text,
    ows_ids pg_catalog.text
);
    

--
-- Name: layers; Type: TABLE; 
--

CREATE TABLE layers (
    id integer NOT NULL,
    class character varying(30),
    active character(1) DEFAULT 't'::bpchar NOT NULL,
    minscaledenom integer,
    resource character varying(50),
    priv_view character varying(50),
    priv_editgeom character varying(50),
    priv_editattr character varying(50),
    name_id integer NOT NULL,
    maxscaledenom integer,
    opacity integer DEFAULT 100,
    filter pg_catalog.text,
    "table" character varying(40),
    edit_maxscale integer,
    snap_maxscale integer,
    default_field character varying(100),
    tolerance integer DEFAULT 5,
    search_scale integer,
    data character varying(200),
    tileindex character varying(200),
    tileitem character varying(30),
    offsite character varying(7),
    history_layer_id integer,
    priv_ows character varying(50),
    label_buffer integer DEFAULT 0,
    label_minfeaturesize integer,
    label_separator character varying(5) DEFAULT '\\n'::character varying,
    label_position integer DEFAULT 110,
    label_font character varying(100),
    parent_id integer,
    path character varying(50),
    label_angle character varying(10) DEFAULT 'auto'::character varying,
    label_backgroundcolor character varying(7),
    label_repeatdistance integer,
    query character(1) DEFAULT 'f'::bpchar,
    srid integer,
    metadata text,
    label_align integer DEFAULT 1,
    label_maxlength integer DEFAULT 0,
    label_priority integer DEFAULT 1,
    label_wrap character varying(1),
    js_file character varying(200),
    js_isbase character(1) DEFAULT 'f'::bpchar,
    classlabel_active character(1) DEFAULT 'f'::bpchar
);


--
-- Name: layers_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE layers_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: layers_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE layers_id_seq OWNED BY layers.id;


--
-- Name: menu_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE menu_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: menu; Type: TABLE; 
--

CREATE TABLE menu (
    id integer DEFAULT nextval('menu_id_seq'::regclass) NOT NULL,
    active character(1) DEFAULT 'f'::bpchar NOT NULL,
    url character varying(200),
    parent_id integer,
    type smallint,
    "order" integer,
    title_id integer,
    text_id integer,
    tags_id integer,
    resource character varying(50),
    privilege character varying(50),
    target character(1) DEFAULT 'f'::bpchar NOT NULL
);


--
-- Name: news_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE news_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: news; Type: TABLE; 
--

CREATE TABLE news (
    id integer DEFAULT nextval('news_id_seq'::regclass) NOT NULL,
    owner_id integer,
    views integer DEFAULT 0,
    image character varying(100),
    active character(1) DEFAULT 'f'::bpchar NOT NULL,
    title_id integer,
    desc_id integer,
    text_id integer,
    tags_id integer,
    created_date timestamp with time zone
);


--
-- Name: roles; Type: TABLE; 
--

CREATE TABLE roles (
    key character varying(50) NOT NULL,
    extends character varying(200),
    "order" integer NOT NULL
);


--
-- Name: roles_order_seq; Type: SEQUENCE;
--

CREATE SEQUENCE roles_order_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: roles_order_seq; Type: SEQUENCE;
--

ALTER SEQUENCE roles_order_seq OWNED BY roles."order";


--
-- Name: rules; Type: TABLE; 
--

CREATE TABLE rules (
    id integer NOT NULL,
    role character varying(50),
    resource character varying(50),
    privilege character varying(50),
    "group" character varying(30)
);


--
-- Name: rules_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE rules_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: rules_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE rules_id_seq OWNED BY rules.id;


--
-- Name: sessions; Type: TABLE; 
--

CREATE TABLE sessions (
    modified integer,
    lifetime integer,
    data pg_catalog.text,
    user_id integer,
    id character varying(32) NOT NULL
);


--
-- Name: styles; Type: TABLE; 
--

CREATE TABLE styles (
    id integer NOT NULL,
    class_id integer,
    color character varying(7),
    width integer,
    outlinecolor character varying(7),
    size integer,
    symbolname character varying(100),
    opacity integer DEFAULT 100 NOT NULL,
    backgroundcolor character varying(7),
    "order" integer,
    angle integer,
    pattern text,
    "position" integer,
    gap double precision,
    linecap integer,
    linejoin integer,
    linejoinmaxsize integer
);


--
-- Name: styles_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE styles_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: styles_id_seq; Type: SEQUENCE;
--

ALTER SEQUENCE styles_id_seq OWNED BY styles.id;


--
-- Name: text_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE text_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: text; Type: TABLE; 
--

CREATE TABLE text (
    id integer DEFAULT nextval('text_id_seq'::regclass) NOT NULL,
    en pg_catalog.text
);


--
-- Name: users_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE users_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- Name: users; Type: TABLE; 
--

CREATE TABLE users (
    id integer DEFAULT nextval('users_id_seq'::regclass) NOT NULL,
    password character varying(32),
    "group" character varying(50),
    fname character varying(30),
    lname character varying(30),
    account character varying(50),
    active character(1) DEFAULT 'f'::bpchar NOT NULL,
    language character(2),
    snap character(1) DEFAULT 't'::bpchar,
    tooltip character(1) DEFAULT 't'::bpchar,
    ip character varying(15),
    signup_date timestamp with time zone,
    lastlogin_date timestamp with time zone,
    expiration_date timestamp with time zone,
    timezone character varying(50),
    email character varying(50),
    email_active character(1) DEFAULT 'f'::bpchar,
    email_secret character varying(32),
    touch_offset integer DEFAULT 3,
    tables pg_catalog.text
);


--
-- Name: mapstats_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE mapstats_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: mapstats; Type: TABLE; 
--

CREATE TABLE mapstats (
    id integer NOT NULL,
    service integer,
    access integer,
    user_id integer,
    date timestamp with time zone,
    layers pg_catalog.text,
    size integer,
    width integer,
    height integer,
    useragent pg_catalog.text,
    ip pg_catalog.text,
    extent public.geometry,
    CONSTRAINT enforce_dims_extent CHECK ((public.ST_NDims(extent) = 2)),
    CONSTRAINT enforce_geotype_extent CHECK (((public.geometrytype(extent) = 'POLYGON'::pg_catalog.text) OR (extent IS NULL)))
);


--
-- Name: querybuilder_queries_id_seq; Type: SEQUENCE;
--

CREATE SEQUENCE querybuilder_queries_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

--
-- Name: querybuilder_queries; Type: TABLE; 
--

CREATE TABLE querybuilder_queries (
    id integer NOT NULL,
    user_id integer,
    query pg_catalog.text,
    "name" pg_catalog.text,
    date timestamp with time zone
);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE base ALTER COLUMN id SET DEFAULT nextval('base_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE classes ALTER COLUMN id SET DEFAULT nextval('classes_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE fields ALTER COLUMN id SET DEFAULT nextval('fields_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE files ALTER COLUMN id SET DEFAULT nextval('files_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE history ALTER COLUMN id SET DEFAULT nextval('history_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE layer_settings ALTER COLUMN id SET DEFAULT nextval('layer_settings_id_seq'::regclass);


--
-- Name: layer_settings_unique; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_settings
    ADD CONSTRAINT layer_settings_unique UNIQUE (layer_id, user_id);
    
    
--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE layers ALTER COLUMN id SET DEFAULT nextval('layers_id_seq'::regclass);


--
-- Name: order; Type: DEFAULT;
--

ALTER TABLE roles ALTER COLUMN "order" SET DEFAULT nextval('roles_order_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE rules ALTER COLUMN id SET DEFAULT nextval('rules_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE styles ALTER COLUMN id SET DEFAULT nextval('styles_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT;
--

ALTER TABLE querybuilder_queries ALTER COLUMN id SET DEFAULT nextval('querybuilder_queries_id_seq'::regclass);


--
-- Name: base_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY base
    ADD CONSTRAINT base_pkey PRIMARY KEY (id);


--
-- Name: classes_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY classes
    ADD CONSTRAINT classes_pkey PRIMARY KEY (id);


--
-- Name: comm_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY comm
    ADD CONSTRAINT comm_pkey PRIMARY KEY (id);


--
-- Name: fields_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY fields
    ADD CONSTRAINT fields_pkey PRIMARY KEY (id);


--
-- Name: files_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY files
    ADD CONSTRAINT files_pkey PRIMARY KEY (id);


--
-- Name: history_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY history
    ADD CONSTRAINT history_pkey PRIMARY KEY (id);


--
-- Name: layer_owsclient_id_key; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_owsclient
    ADD CONSTRAINT layer_owsclient_pkey PRIMARY KEY (id);

    
--
-- Name: layer_settings_id_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_settings
    ADD CONSTRAINT layer_settings_pkey PRIMARY KEY (id);


--
-- Name: layers_labels_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_defaults
    ADD CONSTRAINT layers_labels_pkey PRIMARY KEY (id);


--
-- Name: layer_permissions_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_permissions
    ADD CONSTRAINT layer_permissions_pkey PRIMARY KEY (id);
    

--
-- Name: layers_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layers
    ADD CONSTRAINT layers_pkey PRIMARY KEY (id);


--
-- Name: menu_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY menu
    ADD CONSTRAINT menu_pkey PRIMARY KEY (id);

ALTER TABLE menu CLUSTER ON menu_pkey;


--
-- Name: news_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY news
    ADD CONSTRAINT news_pkey PRIMARY KEY (id);


--
-- Name: publish_layers_id_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY layer_publish
    ADD CONSTRAINT layer_publish_pkey PRIMARY KEY (id);


--
-- Name: roles_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY roles
    ADD CONSTRAINT roles_pkey PRIMARY KEY (key);


--
-- Name: rules_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY rules
    ADD CONSTRAINT rules_pkey PRIMARY KEY (id);


--
-- Name: sessions_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY sessions
    ADD CONSTRAINT sessions_pkey PRIMARY KEY (id);


--
-- Name: styles_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY styles
    ADD CONSTRAINT styles_pkey PRIMARY KEY (id);


--
-- Name: text_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY text
    ADD CONSTRAINT text_pkey PRIMARY KEY (id);


--
-- Name: users_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);
    
    
--
-- Name: mapstats_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY mapstats
    ADD CONSTRAINT mapstats_pkey PRIMARY KEY (id);
    
    
--
-- Name: querybuilder_queries_pkey; Type: CONSTRAINT; 
--

ALTER TABLE ONLY querybuilder_queries
    ADD CONSTRAINT querybuilder_queries_pkey PRIMARY KEY (id);


--
-- Name: history_activate_date_index; Type: INDEX; 
--

CREATE INDEX history_activate_date_index ON history USING btree (activate_date);


--
-- Name: history_active_index; Type: INDEX; 
--

CREATE INDEX history_active_index ON history USING btree (active);


--
-- Name: history_upload_extent_gist; Type: INDEX; 
--

CREATE INDEX history_upload_extent_gist ON history USING gist (upload_extent);


--
-- Name: mapstats_date_index; Type: INDEX; 
--

CREATE INDEX mapstats_date_index ON mapstats USING btree (date);


--
-- Name: mapstats_the_geom_gist; Type: INDEX; 
--

CREATE INDEX mapstats_extent_gist ON mapstats USING gist (extent);


--
-- Name: files_lo_delete; Type: TRIGGER;
--

CREATE TRIGGER files_lo_delete
    BEFORE DELETE ON files
    FOR EACH ROW
    EXECUTE PROCEDURE lo_delete();


--
-- Name: layers_tree_trigger; Type: TRIGGER;
--

CREATE TRIGGER layers_tree_trigger
    BEFORE INSERT OR UPDATE ON layers
    FOR EACH ROW
    EXECUTE PROCEDURE layers_tree();


--
-- Name: classes_layer_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY classes
    ADD CONSTRAINT classes_layer_id_fkey FOREIGN KEY (layer_id) REFERENCES layers(id) ON DELETE CASCADE;


--
-- Name: comm_user_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY comm
    ADD CONSTRAINT comm_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON DELETE CASCADE;


--
-- Name: fields_layer_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY fields
    ADD CONSTRAINT fields_layer_id_fkey FOREIGN KEY (layer_id) REFERENCES layers(id) ON DELETE CASCADE;
    

--
-- Name: joins_layer_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY joins
    ADD CONSTRAINT joins_layer_id_fkey FOREIGN KEY (layer_id) REFERENCES layers(id) ON DELETE CASCADE;

    
--
-- Name: layer_defaults_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layer_defaults
    ADD CONSTRAINT layer_defaults_id_fkey FOREIGN KEY (id) REFERENCES layers(id) ON DELETE CASCADE;


--
-- Name: layers_parent_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layers
    ADD CONSTRAINT layers_parent_id_fkey FOREIGN KEY (parent_id) REFERENCES layers(id) ON DELETE CASCADE;


--
-- Name: ls_layer_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layer_settings
    ADD CONSTRAINT layer_settings_layer_id_fkey FOREIGN KEY (layer_id) REFERENCES layers(id) ON DELETE CASCADE;


--
-- Name: ls_user_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layer_settings
    ADD CONSTRAINT layer_settings_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON DELETE CASCADE;
    
    
--
-- Name: layer_permissions_id_fkey; Type: FK CONSTRAINT;
--
ALTER TABLE ONLY layer_permissions
    ADD CONSTRAINT layer_permissions_id_fkey FOREIGN KEY (id) REFERENCES layers(id) ON DELETE CASCADE;

    
--
-- Name: rules_role_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY rules
    ADD CONSTRAINT rules_role_fkey FOREIGN KEY (role) REFERENCES roles(key) ON DELETE CASCADE;


--
-- Name: styles_class_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY styles
    ADD CONSTRAINT styles_class_id_fkey FOREIGN KEY (class_id) REFERENCES classes(id) ON DELETE CASCADE;


--
-- Name: layer_owsclient_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layer_owsclient
    ADD CONSTRAINT layer_owsclient_id_fkey FOREIGN KEY (id) REFERENCES layers(id) ON DELETE CASCADE;


--
-- Name: layer_publish_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY layer_publish
    ADD CONSTRAINT layer_publish_id_fkey FOREIGN KEY (id) REFERENCES layers(id) ON DELETE CASCADE;
    

--
-- Name: sessions_user_id_fkey; Type: FK CONSTRAINT;
--

ALTER TABLE ONLY sessions
    ADD CONSTRAINT sessions_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON DELETE CASCADE;