<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Acl.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Acl
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Acl
 */
require_once 'Zend/Acl.php';

/**
 * Access control lists
 * 
 * @category   Giscuit
 * @package    Giscuit_Acl
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Acl extends Zend_Acl
{

    /**
     * Extended roles separator
     * 
     * @var string
     */
    protected $_separator = ',';

    /**
     * Populate acl with roles 
     * 
     * @return void
     */
    public function setRoles ()
    {
        $rolesTable = new Giscuit_Db_Table_Roles();
        $roles = $rolesTable->getItems();
        
        foreach ($roles as $role) {
            if (empty($role->extends)) {
                $this->addRole(new Zend_Acl_Role($role->key));
            } else {
                $this->addRole(new Zend_Acl_Role($role->key), explode($this->_separator, $role->extends));
            }
        }
    
    }

    /**
     * Populate acl with rules
     * 
     * @return void
     */
    public function setRules ()
    {
        $rulesTable = new Giscuit_Db_Table_Rules();
        $rules = $rulesTable->getItems();
        
        foreach ($rules as $rule) {
            
            //Skip empty rules
            if (empty($rule->role) || empty($rule->resource))
                continue;
                
            //Add resource
            if (! $this->has($rule->resource)) {
                $this->add(new Zend_Acl_Resource($rule->resource));
            }
            
            //Add rule
            if (empty($rule->privilege)) {
                $this->allow($rule->role, $rule->resource);
            } else {
                $this->allow($rule->role, $rule->resource, $rule->privilege);
            }
        }
    }
    
    /**
     * Clear cache
     */
    public static function clearCache()
    {
        //Clear cache
        if (Zend_Registry::isRegistered('cache')) {
            Zend_Registry::get('cache')->remove(md5(GT_APP_PATH) . '_acl');
        }
    }

}
