<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Ows.php 2844 2012-07-09 08:54:35Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Action
 */
require_once 'Zend/Controller/Action.php';

/**
 * Abstract OWS service controller  
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @copyright  Copyright (c) 2012 VEC
 */
abstract class Giscuit_Controller_Ows extends Zend_Controller_Action
{
    /**
     * Parse the request object
     * 
     * @param ms_newowsrequestobj $request
     * @return void
     */
    abstract protected function _parseRequest($request);
    
    /**
     * Layer settings
     * 
     * @var array
     */
    protected $_settings;
    
    /**
     * Map
     * 
     * @var Giscuit_Map_Map
     */
    protected $_map;

    /**
     * Request
     * 
     * @var ms_newowsrequestobj
     */
    protected $_request;
    
    /**
     * Service id to name
     * 
     * @var array
     */
    public static $serviceNames = array (
        '1' => 'wms' ,
        '2' => 'wfs' ,
        '3' => 'web' ,
        '4' => 'tilecache' ,
        '5' => 'offline' ,
        '6' => 'wcs'
    );
    
    /**
     * Access id to name
     *  
     * @var array
     */
    public static $accessNames = array (
        '1' => 'public' ,
        '2' => 'private'
    );
    
    /**
     * Tilecache action
     * 
     * @return void
     */
    public function tilecacheAction ()
    {
        $config = Zend_Registry::get('config');
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        //Check if the request for tile generation was made from tilecache
        if($_GET['server_origin'] != $config->server_origin) {
            throw new Exception("Invalid server_origin string. Update your tilecache.cfg with a proper server_origin string.");    
        }
        
        $this->_addMap();
        $this->_addLayers('tilecache');
        $this->_formatLayers();
        $this->_draw('tilecache');
    }
    
    /**
     * Offline action
     * 
     * @return void
     */
    public function offlineAction ()
    {
        $config = Zend_Registry::get('config');
        
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        //Check if the request for tile generation was made from tilecache
        if($_GET['server_origin'] != $config->server_origin) {
            throw new Exception('Invalid server_origin string. Update your tilecache.cfg with a proper server_origin string.');    
        }
        
        $this->_addMap();
        $this->_addLayers('offline');
        $this->_formatLayers();
        $this->_draw('offline');
    }

    /**
     * Private action
     * 
     * @return void
     */
    public function privateAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
        
        $translate = Zend_Registry::get('Zend_Translate');
        
        //Request
        $request = $this->getRequest();
        $account = $request->getParam('account');
        $password = $request->getParam('password');

        //HTTP Authentication fix with PHP running as CGI/SuExec
        if(! isset($_SERVER['PHP_AUTH_USER']) && ! isset($_SERVER['PHP_AUTH_PW'])) {
            if(! empty($_SERVER['REDIRECT_REDIRECT_HTTP_AUTHORIZATION'])) {
                list($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']) = explode(':', base64_decode(substr($_SERVER['REDIRECT_REDIRECT_HTTP_AUTHORIZATION'], 6))); 
            }
            if(! empty($_SERVER['REDIRECT_HTTP_AUTHORIZATION'])) {
                list($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW']) = explode(':', base64_decode(substr($_SERVER['REDIRECT_HTTP_AUTHORIZATION'], 6))); 
            }
        }

        if (! empty($account) && ! empty($password)) {
            $this->_authenticate($account, $password, true);
        } else {
            if (! isset($_SERVER['PHP_AUTH_USER'])) {
                header('WWW-Authenticate: Basic realm="My Realm"');
                header('HTTP/1.0 401 Unauthorized');
                
                die($translate->_('authenticationRequired'));
            } else {
                $this->_authenticate($_SERVER['PHP_AUTH_USER'], $_SERVER['PHP_AUTH_PW'], false);
            }
        }
        
        //Get user id
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        
        //Add map
        $this->_addMap();
                        
        $this->_addLayers('private');
        $this->_formatLayers();
        $this->_draw('private');
    }

    /**
     * Public action
     * 
     * @return void
     */
    public function publicAction ()
    {
        $layout = Zend_Layout::getMvcInstance();
        if ($layout instanceof Zend_Layout) {
            $layout->disableLayout();
        }
        $this->_helper->viewRenderer->setNoRender();
                
        $this->_addMap();
        $this->_addLayers('public');
        $this->_formatLayers();
        $this->_draw('public');
    }

    
    /**
     * Initialize map
     */
    protected function _addMap ()
    {
        $config = Zend_Registry::get('config');
        $locale = Zend_Registry::get('Zend_Translate')->getLocale();
        
        $urlHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('url');
                
        $this->_request = $this->_getRequest();
        
        $this->_map = new Giscuit_Map_Map(GT_CONFIG_PATH . DIR_SEP . 'default.map');
        
        $this->_map->setParamsFromConfig();
        
        $this->_map->setMetaData("wms_getmap_formatlist", "image/jpeg,image/png,image/png; mode=24bit");
        $this->_map->setMetaData('ows_encoding', 'UTF-8');
        $this->_map->setMetaData('ows_enable_request', '*');
        
        //Metadata
        if(isset($config->ows->$locale) && ($config->ows->$locale instanceof Zend_Config)) {
            $metadata = $config->ows->$locale->toArray();
        
            foreach ($metadata as $key => $value) {
                $this->_map->setMetaData($key, $value);
            }
        }
        
        $this->_map->setMetaData('ows_srs', $config->ows->ows_srs);
        
        //WCS alias
        $this->_map->setMetaData('wcs_label', $config->ows->$locale->ows_title);
        
        //Online resource
        if(GT_TESTING == false) {
            //Query string
            $queryString = '';
            if (! empty($_SERVER['PHP_AUTH_USER']) && ! empty($_SERVER['PHP_AUTH_PW'])) {
                $queryString .= '&account=' . $_SERVER['PHP_AUTH_USER'] . '&password=' . md5($_SERVER['PHP_AUTH_PW']);
            }
            
            $owsOnlineresource = $this->_map->getMetaData('ows_onlineresource');
            if($owsOnlineresource) {
                $owsOnlineresource = $owsOnlineresource . $queryString;
                foreach ($_SERVER as $key => $value) {
                    $owsOnlineresource = str_replace('$' . $key . '$', $value, $owsOnlineresource);
                }
                $this->_map->setMetaData("ows_onlineresource", $owsOnlineresource);
                
            } else {
                $this->_map->setMetaData("ows_onlineresource", 
                    'http://' . $_SERVER['SERVER_NAME'] . ':' . 
                    $_SERVER['SERVER_PORT'] . 
                    $_SERVER['PATH_INFO'] . 
                    '?' . $queryString
                );
            }
        } else {
            $this->_map->setMetaData("ows_onlineresource", 'http://giscuit.com');
        }
        
    }

    
    /**
     * Authenticate
     * 
     * @param string $account User account name
     * @param string $password User password
     * @param bool $md5Credential Is password already encrypted with md5
     */
    protected function _authenticate ($account, $password, $md5Credential = false)
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $form = new Giscuit_Form_Login();
        
        $authValues = array(
            'account' => $account , 
            'password' => $password
        );
        
        if ($form->isValid($authValues)) {
            
            $authError = Giscuit_Controller_Plugin_Acl::authenticate($authValues, array(
                'nonPersistent' => true ,
                'md5Credential' => $md5Credential
            ));
            
            if (empty($authError)) {

            } else {
                $this->triggerError($authError);
            }
        
        } else {
            $this->triggerError($translate->_('invalidInput'));
        }
    
    }

    /**
     * Add current layer to the map object
     * 
     * @param string $type <ul>Available types:
     * <li>public - Add public layers</li>
     * <li>private - Add layers based on current authenticated user</li>
     * <li>tilecache - Add published layers</li></ul>
     * @return void
     */
    protected function _addLayers ($type)
    {   
        if ($type == 'public') {
            $options = $this->_parseLayerOptions(array(
                'addEditLayer' => true , 
                'orderDirection' => 'desc' , 
                'whereActive' => 't' , 
                'withFields' => true , 
                'withClassesAndStyles' => true , 
                'withPublish' => true , 
                'withGroups' => true ,
                'withMetadata' => true
            ));
        } elseif ($type == 'private') {
            $options = $this->_parseLayerOptions(array(
                'addEditLayer' => true , 
                'orderDirection' => 'desc' , 
                'whereActive' => 't' , 
                'withFields' => true , 
                'withClassesAndStyles' => true , 
                'withOwsAcl' => true , 
                'withPublish' => true , 
                'withGroups' => true ,
                'withMetadata' => true
            ));
        } elseif ($type == 'tilecache') {
            $options = Giscuit_Map_Map::getPublishOptions();
            $options['withGroups'] = true;
        } elseif ($type == 'offline') {
            $options = $this->_parseLayerOptions(array(
                'addEditLayer' => true , 
                'orderDirection' => 'desc' , 
                'whereActive' => 't' , 
                'withFields' => true , 
                'withClassesAndStyles' => true , 
            ));
        } else {
            throw new Exception('Invalid type');
        }
        
        if (strtolower($this->_request->getValueByName('SERVICE')) == 'wms' &&
            strtolower($this->_request->getValueByName('REQUEST')) == 'getfeatureinfo' ) {
                
            $this->_map->dumpAllAttributes = true;
            $options['withClassesAndStyles'] = false;
        }
        
        if (strtolower($this->_request->getValueByName('SERVICE')) == 'wfs') {
            $options['withClassesAndStyles'] = false;
        }
                
        $this->_settings = $this->_map->addCurrentLayers($options);
        
    }

    /**
     * Set proper metadata and propreties for layers 
     * 
     * @return void
     */
    protected function _formatLayers ()
    {
        $config = Zend_Registry::get('config');
        
        //Get extent
        $extent = $config->map->extent->toArray();
        $extent = "{$extent['min']['x']} {$extent['min']['y']} {$extent['max']['x']} {$extent['max']['y']}";
        
        foreach ($this->_map->children as $layer) {

            $settings = $layer->getSettings();
            
            if (! isset($settings['active']) || $settings['active'] == 'f') {
                continue;
            }
            
            $layer->setMetaData("ows_layer_group", $this->_getPath($settings['path']));
            
            //Set metadata
            if(is_array($settings['metadata'])) {
                foreach ($settings['metadata'] as $key => $value) {
                    $layer->setMetaData($key, $value);
                }
            }
            
            //Vector
            if (is_subclass_of('Giscuit_Map_Layer_' . $settings['class'], 'Giscuit_Map_Layer_Vector')) {
                $layer->setProcessing('LABELCACHE_MAP_EDGE_BUFFER=-10');
                
                $layer->set("template", GT_CONFIG_PATH . DIR_SEP . 'blank.html');
                $layer->set("dump", MS_TRUE);
                
                //Wms
                $layer->setMetaData("ows_include_items", "all");
                //FIXME: Use tables primary key
                $layer->setMetaData("ows_featureid", "gid");
                
                //Wfs
                $layer->setMetaData("gml_include_items", "all");
                //FIXME: Use tables primary key
                $layer->setMetaData("gml_featureid", "gid");
                
            //Raster    
            } elseif (is_subclass_of('Giscuit_Map_Layer_' . $settings['class'], 'Giscuit_Map_Layer_Raster')) {
                if($settings['query'] == 't') {
                    $layer->set("template", GT_CONFIG_PATH . DIR_SEP . 'blank.html');
                    $layer->setMetaData("ows_include_items", "value_list");    
                }
            }
            
            $layer->setMetaData("ows_title", $settings['name']);

            //WCS alias
            $layer->setMetaData('wcs_label', $settings['name']);
            
            //Default extent
            if (empty($settings['metadata']['ows_extent'])) {
                $layer->setMetaData("ows_extent", $extent);
            }
        
        }
    
    }

    /**
     * Dispatch the result
     * 
     * @return void
     */
    protected function _draw ($type)
    {
        //Flush
        if(GT_TESTING == false) {
            while (ob_get_level()) {
                ob_end_clean();
            }
        }
        
        ms_ioinstallstdouttobuffer();
        
        $this->_map->owsdispatch($this->_request);
        
        $contentType = ms_iostripstdoutbuffercontenttype();
        
        if(GT_TESTING == false) {
            header("Content-type: $contentType");
        }
        
        //FIXME: Remove POST element from wfs getcapabilities response
        if (strtolower($this->_request->getValueByName('REQUEST')) == 'getcapabilities' 
            && strtolower($this->_request->getValueByName('SERVICE')) == 'wfs' 
            && strtolower($this->_request->getValueByName('VERSION')) == '1.0.0') {
            
            $result = ms_iogetstdoutbufferstring();
            $result = preg_replace('/\<Post onlineResource="(.*?)" \/\>/i', NULL, $result);
            
            $size = strlen($result);
            
            echo $result;
        } else {
            $size = ms_iogetStdoutBufferBytes();
        }
        
        ms_ioresethandlers();
        
        $service = strtolower($this->_request->getValueByName('SERVICE'));
    
        //Access
        if($type == 'tilecache') {
            $service = 'tilecache';
            $access = 'public';
        } else if ($type == 'offline') {
            $service = 'offline';
            $access = 'private';
        } else {
            $access = $type;
        }
        
        //Layers
        if($service == 'wms' || $service == 'tilecache' || $service == 'offline' || $service == 'wcs') {
            $layers = $this->_request->getValueByName('LAYERS');
        } elseif($service == 'wfs') {
            $layers = $this->_request->getValueByName('TYPENAME');
        } 
        
        if(! empty($access) && ! empty($service)) {
            self::log($service, $access, $size, $layers, $this->_map->getExtent(), $this->_map->getSize());
        }
        
        if(GT_TESTING == false) {
            exit;
        }
    }

    /**
     * Add the request parameters to the request object
     * 
     * @return ms_newowsrequestobj
     */
    protected function _getRequest ()
    {
        $request = ms_newowsrequestobj();
        
        foreach ($this->getRequest()->getParams() as $key => $value) {
            $request->setParameter(strtoupper($key), $value);
        }
        
        //FIXME: No exception parameter is provided when using wms 1.3.0
        if ($request->getValueByName('VERSION') == '1.3.0' && 
            ($request->getValueByName('EXCEPTIONS') == '' || strtolower($request->getValueByName('EXCEPTIONS')) == 'application/vnd.ogc.se_xml')
        ) {
            
            $request->setParameter('EXCEPTIONS', 'XML');
        }
        
        //FIXME: Force WMS version 1.1.1 for MapInfo
        if (isset($_SERVER['HTTP_USER_AGENT']) && 
            stristr($_SERVER['HTTP_USER_AGENT'], 'MapInfow.exe') &&
            $this->getRequest()->getControllerName() == 'wms' && 
            strtolower($request->getValueByName('REQUEST')) == 'getcapabilities') {
                
            $request->setParameter('VERSION', '1.1.1');   
        }
            
        $this->_parseRequest($request);
        
        return $request;
    }

    /**
     * Parse layer options
     * 
     * @param array $options <ul>Available options:
     * @return array
     */
    protected function _parseLayerOptions (array $options)
    {
        return $options;
    }

    /**
     * Get the layer path
     * 
     * @param string $path
     * @return string
     */
    protected function _getPath ($path)
    {
        if (empty($path) || $path == '/') {
            return '';
        }
        
        $path = trim($path, '/');
        $path = explode('/', $path);
        unset($path[count($path) - 1]);
        
        foreach ($path as $key => $value) {
            if(isset($this->_settings[$path[$key]])) {
                $path[$key] = $this->_settings[$path[$key]]['name'];
            }
        }
        
        $path = '/' . implode('/', $path);
        
        if ($path == '/') {
            return '';
        }
        
        return $path;
    }

    /**
     * Trigger an OWS error
     * 
     * @param string $text Error message
     * @param string $code Error code
     * @return void
     */
    public function triggerError ($text, $code = '')
    {
        die($text);
    }
    
    /**
     * Log ows request
     * 
     * @param string $type
     * @param integer $size
     * @return void 
     */
    public static function log($service, $access, $size, $layers, $extent, $imageSize) 
    {
        $mapstatsTable = new Giscuit_Db_Table_Mapstats();
        $data = array();

        //Protocol
        foreach(self::$serviceNames as $key => $value) {
            if($service == $value) {
                $data['service'] = $key;
                break;
            }    
        }
        if(! isset($data['service'])) {
            throw new Giscuit_Exception("Invalid service ({$service})");
        }
        
        //Access
        if ($access == 'public') {
            $data['access'] = 1;
        } else if ($access == 'private') {
            $data['access'] = 2;
            $data['user_id'] = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        }
        
        $data['layers'] = $layers;
        $data['size'] = $size;
        $data['extent'] = $extent;
        $data['imageSize'] = $imageSize;
        
        $mapstatsTable->addItem($data);
        
    }
}