<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for layer masking
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Masking extends Giscuit_Controller_Plugin_Abstract
{

    protected $_translations = array(
        'en' => array(
            'geometryOutsideAllowedZone' => 'Geometry is outside of allowed zone'
        ),
        'ro' => array(
            'geometryOutsideAllowedZone' => 'Geometria este în afara zonei permise'
        ),
        'ru' => array(
            'geometryOutsideAllowedZone' => 'Геометрия находится за пределами разрешенной зоны'
        )
    );
    
    public $layerClass = 'Masking';
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        $rolesTable = new Giscuit_Db_Table_Roles();
        $layersTable = new Giscuit_Db_Table_Layers();

        //Insert rules and roles for layer access
        $maskingRole = new stdClass();
        $maskingRole->key = 'masking';
        $rolesTable->addItem($maskingRole);
        
        $data = array('role' => 'admin', 'resource' => 'masking_layer', 'privilege' => 'view', 'group' => 'layer');
        $rulesTable->insert($data);
        
        $data = array('role' => 'admin', 'resource' => 'masking_layer', 'privilege' => 'edit', 'group' => 'layer');
        $rulesTable->insert($data);
        
        $data = array('role' => 'masking', 'resource' => 'masking', 'privilege' => '', 'group' => '');
        $rulesTable->insert($data);
        
        //Create mask table
        $schema = $newConfig->production->database->schema;
        $layersSchema = $schema . '_layers';
        $db->beginTransaction();
        try {
            $db->query("
            CREATE TABLE {$layersSchema}.masking (
                gid serial,
                role_view character varying(50),
                role_edit character varying(50),
                the_geom public.geometry,
                CONSTRAINT enforce_dims_the_geom CHECK ((public.st_ndims(the_geom) = 2)),
                CONSTRAINT enforce_geotype_the_geom CHECK (((public.geometrytype(the_geom) = 'MULTIPOLYGON'::text) OR (the_geom IS NULL)))
            );");

            $db->query("ALTER TABLE ONLY {$layersSchema}.masking ADD CONSTRAINT masking_pkey PRIMARY KEY (gid);");

            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Create layer
        $form = new Giscuit_Form_Admin_LayersAdd();
        $form->getElement('class')->setValue($this->layerClass);
        $form->getElement('name_en')->setValue('Masking');
        $form->getElement('table')->setValue('masking');
        $layerId = $layersTable->add($form, array(
            'type' => 'vector' , 
            'layerType' => 'Polygon' ,
            'parentId' => ''
        ));
        
        
        $db->beginTransaction();
        try {
            $db->query("UPDATE {$schema}.layers SET active = 't', edit_maxscale = 50000, resource = 'masking_layer', priv_view = 'view', priv_editgeom = 'edit' WHERE id = {$layerId};");
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        $db = Zend_Registry::get('db');
        $rulesTable = new Giscuit_Db_Table_Rules();
        $rolesTable = new Giscuit_Db_Table_Roles();
        
        //Delete rules and roles
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'masking_layer');
        $rulesTable->delete($where);
        
        $where = $rulesTable->getAdapter()->quoteInto('resource = ?', 'masking');
        $rulesTable->delete($where);
        
        $where = $rolesTable->getAdapter()->quoteInto('key = ?', 'masking');
        $rolesTable->delete($where);
        $rolesTable->cleanupRoles();
        Giscuit_Acl::clearCache();
        
        //Drop tables
        $schema = $newConfig->production->database->schema;
        $db->beginTransaction();
        try {
            $db->query("DROP TABLE IF EXISTS {$schema}_layers.masking CASCADE;");
            
            $db->commit();
        } catch (Exception $e) {
            $db->rollBack();
            throw new Giscuit_Db_Table_Exception($e->getMessage());
        }
        
        //Delete layer and it's files
        $layersTable = new Giscuit_Db_Table_Layers();
        $layerData = $layersTable->getItems(array('whereClass' => $this->layerClass));
        
        reset($layerData);
        $layerId = key($layerData);
        
        
        if ($layerId != NULL) {
            $layersTable->delete('id = ' . $layerId);
        }
        
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    public function getCurrentMasks($roleType, $geometry)
    {
        $config = Zend_Registry::get('config');
        $acl = Zend_Registry::get('acl');
        $userInfo = Zend_Auth::getInstance()->getIdentity();
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');

        $tableName = 'Giscuit_Db_Table_Layer_' . $this->layerClass;
        $layerTable = new $tableName(array(
            'name' => 'masking'
        ));
        
        //Check if user has any restrictions at all
        if ($isAllowed->direct('admin') || ! $isAllowed->direct('masking')) {
            return false;
        }
        
        //Get all allowed zones
        $select = $layerTable->getSelect('gid_geometry', array(
            'geometryAsWkt' => true,
            'whereIntersection' => "ST_AsText('{$geometry}')",
            'fields' => array(
                'role_view',
                'role_edit'
            )
        ));
        $rows = $layerTable->fetchAll($select)->toArray();
    
        //Filter results by user role
        if (! $isAllowed->direct('admin')) {
            foreach ($rows as $key => $value) {
                if ($userInfo->group != $value[$roleType] && ! $acl->inheritsRole($userInfo->group, $value[$roleType])) {
                    unset($rows[$key]);
                }
            }
        }
        
        return $rows;
    }
    
    public function whereIntersections ($select, $roleType, $tableName, $map)
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');

        if ($isAllowed->direct('admin') || empty($select)) {
            return false;
        }
        
        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');

        $theGeomField = $db->quoteIdentifier($tableName . '.the_geom');
        $geometry = Giscuit_Map_Map::extentToWktPolygon($map->getExtent());
        $masksResult = $this->getCurrentMasks($roleType, $geometry);
                
        if ($masksResult !== false && count($masksResult) == 0) {
            //FIXME: return no geometries if there are no masks
            $select->where('false = true');
        } else if ($masksResult !== false && count($masksResult) != 0) {
            $whereArray = array();
            foreach ($masksResult as $mask) {
                $whereArray[] = "ST_Intersects(" . $theGeomField . ", ST_GeometryFromText('{$mask['the_geom']}', {$config->map->srid}))";
            }
            $where = '(' . implode(' OR ', $whereArray) . ')';
            $select->where($where);
        }
        
        return true;
    }
    
    public function isAllowedByGid ($table, $gid)
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $geometryInZone = true;
        
        if (! $isAllowed->direct('admin')) {
            $geometryInZone = false;
            $select = $table->getSelect('gid_geometry', array(
                'geometryAsWkt' => true
            ));
            $select->where('gid = ?', $gid);
            $result = $table->fetchRow($select);
            
            $masksResult = $this->getCurrentMasks('role_edit', $result->the_geom);
            if ($masksResult === false || count($masksResult) != 0) {
                $geometryInZone = true;
            }
        }

        return $geometryInZone;
    }
    
    public function isAllowedByGeometry ($geometry) 
    {
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $geometryInZone = true;
        
        if (! $isAllowed->direct('admin')) {
            $geometryInZone = false;
            $masksResult = $this->getCurrentMasks('role_edit', $geometry);
            if ($masksResult === false || count($masksResult) != 0) {
                $geometryInZone = true;
            }
        }
        
        return $geometryInZone;
    }

}