<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Controller_Plugin_Abstract
 */
require_once 'Zend/Controller/Plugin/Abstract.php';

/**
 * Plugin for serial communication
 *  
 * @category   Giscuit
 * @package    Giscuit_Controller
 * @subpackage Plugin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Controller_Plugin_Serial extends Giscuit_Controller_Plugin_Abstract
{
    
    protected $_translations = array(
        'en' => array(
            'devices' => 'Devices' ,
            'baudrate' => 'Baud rate' ,
            'connect' => 'Connect'
        ),
        'ro' => array(
            'devices' => 'Dispozitive' ,
            'baudrate' => 'Rata de transfer' ,
            'connect' => 'Conectare'
        ),
        'ru' => array(
            'devices' => 'Устройства' ,
            'baudrate' => 'Скорость передачи' ,
            'connect' => 'Подключится'
        )
    );
    
    /**
     * List of the javascript files to include
     * 
     * @return array
     */
    public function getJsFilesList ()
    {
        return array(
            'Control/Serial.js' 
        );
    }
    
    /**
     * List of the javascript files to include for mobile
     * 
     * @return array
     */
    public function getMobileJsFilesList ()
    {
        return array(
            'Control/Serial.js',
            'Mobile/Control/Serial.js' 
        );
    }
    
    /**
     * Install plugin
     * 
     * @return boolean
     */
    public function install($newConfig) 
    {
        //Add default section translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
        
            $newMessages = array_merge($translate->getMessages(), $languageValue);
            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $newMessages);
            
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }
    
    /**
     * Uninstall plugin
     *  
     * @return boolean
     */
    public function uninstall($newConfig) 
    {
        //Delete translations
        foreach ($this->_translations as $languageKey => $languageValue) {
            $languageFile = GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx';
            if(! file_exists($languageFile)) {
                continue;       
            }
            
            $translate = new Zend_Translate('tmx', $languageFile, $languageKey);
            
            $translations = $translate->getMessages();
            foreach ($languageValue as $key => $value) {
                unset($translations[$key]);
            }

            $newTmx = Giscuit_Controller_Plugin_Abstract::arrayToTmx($languageKey, $translations);
            file_put_contents(GT_LANGUAGES_PATH . DIR_SEP . 'default.' . $languageKey . '.tmx', $newTmx);
        }
    }

}