<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Menu.php 1559 2011-08-03 10:57:49Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * @see Giscuit_Db_Table_Multilang
 */
require_once 'Giscuit/Db/Table/Multilang.php';

/**
 * Class for incidents categories table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */class Giscuit_Db_Table_Layer_IncidentsCategory extends Giscuit_Db_Table_Multilang
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'incidents_category';
    
    /**
     * Contains the names of multilang fields inside the table
     * 
     * @var array
     */
    protected $_multilangTextColumns = array(
        'name'
    );
    
    /**
     * Constructor
     *
     * @return void
     */
    public function __construct ($options = array())
    {
        $config = Zend_Registry::get('config');

        $this->_schema = $config->database->schema . '_layers';
        $this->_mainSchema = $config->database->schema;
                
        parent::__construct($options);
    }

    /**
     * Add item
     * 
     * @param Zend_Form $data A form object populated with request data
     * @param array $options <ul>Available options:
     * <li>parent_id integer - Parent menu section id</li></ul>
     * @return $data array - id and name_id of the new element
     */
    public function addItem (array $options = array())
    {
        $newRow = $this->createRow();

        $textTable = new Giscuit_Db_Table_Text();
        $newTextRow = $textTable->createRow();
        
        $newRow->name_id = $newTextRow->save();
        $data['name_id'] = $newRow->name_id;
        
        $data['id'] = $newRow->save();
        
        return $data;
    }

    /**
     * Get items
     * 
     * @param array $options <ul>Available options:
     * <li>withName bool - Include name field in the results</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'color'
        );
        
        $select->from($this, $fields);
        
        //Fetch text
        if (! empty($options['withName'])) {
            $this->joinMultilangField($select, 'name', null, $this->_mainSchema);
        }
        
        if (! empty($options['withAllNames'])) {
            $textColumns = array_keys(Zend_Registry::get('config')->language->toArray());
            $textColumns['name_id'] = 'id';
            
            $select->joinLeft(array(
                'text' => $this->_mainSchema . '.text'
            ), "text.id = {$this->_name}.name_id", $textColumns);
        }

        $result = $this->fetchAll($select);
        
        return $result;
    }
    
    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $config = Zend_Registry::get('config');
        
        $rows = $this->find($id);
        
        if ($rows->count() == 0) {
            throw new Giscuit_Exception("Invalid category id :" . $id);
        }
        
        $row = $rows->current();
        
        //Get column
        $column = $data->column;

        $row->$column = $data->data;
        
        return $row->save();
    }
    
}
