<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Fields.php 2897 2012-08-02 11:41:08Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for queries table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_QuerybuilderQueries extends Giscuit_Db_Table_Abstract
{

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'querybuilder_queries';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'id';

    /**
     * Get queries
     * 
     * @param array $options Available options:
     * <ul><li>whereStm string - Fetch rows that match the statement</li>
     * <li>whereLayerId integer - Fetch rows that match the layer id</li>
     * <li>asArray bool - Get results as array</li></ul>
     * @return Zend_Db_Table_Rowset
     */
    public function getItems (array $options = array())
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $fields = array(
            'id' , 
            'user_id' , 
            'query' ,
            'name' ,
            'date'
        );
        
        $select->from($this, $fields);
        
        //Where user id
        if (isset($options['whereUserId'])) {
            $select->where($this->_name . '.user_id = ?', $options['whereUserId']);
        }
        
        //Where query id
        $this->whereId($select, $options);
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Add item
     * 
     * @param array $data Query data
     * @return Zend_Db_Table_Row
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        $newRow->user_id = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct();
        $newRow->query = $data['query'];
        $newRow->name = $data['name'];
        $newRow->date = new Zend_Db_Expr('now()');
        
        return $newRow->save();
    }

}
