<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Roles.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Db_Table_Abstract
 */
require_once 'Giscuit/Db/Table/Abstract.php';

/**
 * Class for roles table 
 *
 * @category   Giscuit
 * @package    Giscuit_Db_Table
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Db_Table_Roles extends Giscuit_Db_Table_Abstract
{

    /**
     * Extended roles separator
     * 
     * @var string
     */
    protected $_separator = ',';

    /**
     * Table name
     * 
     * @var string
     */
    protected $_name = 'roles';

    /**
     * Table primary key name
     * 
     * @var string
     */
    protected $_primary = 'key';

    /**
     * Get roles
     * 
     * @return Zend_Db_Table_Rowset
     */
    public function getItems ()
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);
        
        $select->order($this->_name . '.order asc');
        
        $result = $this->fetchAll($select);
        
        return $result;
    }

    /**
     * Update item
     * 
     * @param integer $id The id of the row to edit 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the element
     */
    public function editItem ($id, $data)
    {
        $rows = $this->find($id);
        $row = $rows->current();
        
        if ($data->column == 'extends' && $data->data != $id) {
            //Remove key from extends
            $data->data = str_replace(',' . $id, NULL, $data->data);
            $row->extends = $data->data;
        }
        
        $result = $row->save();
        
        $this->cleanupRoles();
        Giscuit_Acl::clearCache();
        
        return $result;
    
    }

    /**
     * Remove orphan roles and apply proper order
     * 
     * @return void
     */
    public function cleanupRoles ()
    {
        //Remove orphan extends
        $currRolesArr = array();
        $currRoles = $this->getItems();
        foreach ($currRoles as $role) {
            $currRolesArr[] = $role->key;
        }
        
        foreach ($currRoles as $role) {
            if (! empty($role->extends)) {
                $extendsArr = explode($this->_separator, $role->extends);
                foreach ($extendsArr as $childKey => $child) {
                    if (! in_array($child, $currRolesArr)) {
                        unset($extendsArr[$childKey]);
                        
                        $rows = $this->find($role->key);
                        $row = $rows->current();
                        $row->extends = implode($this->_separator, $extendsArr);
                        
                        $row->save();
                    }
                }
            }
        }
        
        //Order
        $roles = $this->getItems()->toArray();
        
        $finished = false;
        $a = 0;
        
        while ($finished == false) {
            $a ++;
            $initializedArr = array();
            $b = 0;
            foreach ($roles as $roleKey => $roleValue) {
                if (empty($roleValue['extends'])) {
                    $initializedArr[] = $roleValue['key'];
                } else {
                    $allParentsInitialized = true;
                    
                    $extends = explode($this->_separator, $roleValue['extends']);
                    
                    foreach ($extends as $extend) {
                        if (! in_array($extend, $initializedArr)) {
                            $roles[] = $roleValue;
                            unset($roles[$roleKey]);
                            $allParentsInitialized = false;
                            break;
                        }
                    }
                    
                    if ($allParentsInitialized == true) {
                        $initializedArr[] = $roleValue['key'];
                    } else {
                        break;
                    }
                }
                
                if ($b + 1 == count($roles)) {
                    $finished = true;
                }
                
                $b ++;
            }
            
            if ($a > 99) {
                throw new Exception('Invalid roles hierarchy');

            }
        }
        
        //Save new order        
        $roles = array_values($roles);
        foreach ($roles as $key => $value) {
            $rows = $this->find($value['key']);
            $row = $rows->current();
            $row->order = $key + 1;
                        
            $row->save();
        }
    
    }

    /**
     * Add item
     * 
     * @param Zend_Filter_Input $data A Zend_Filter_Input object populated with request data
     * @return integer The id of the new element
     */
    public function addItem ($data)
    {
        $newRow = $this->createRow();
        
        $newRow->key = $data->key;
        
        $result = $newRow->save();
        
        $this->cleanupRoles();
        Giscuit_Acl::clearCache();
        
        return $result;
    }
    
}
