<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: DatabaseUpload.php 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Upload database table form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_DatabaseUpload extends Giscuit_Form_Abstract
{

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        //Name
        $this->addElement('text', 'name', array(
            'filters' => array(
                'StringToLower'
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                ) , 
                array(
                    'TablenameAvailable'
                ) ,
                new Zend_Validate_Regex('/^([0-9a-z_])+$/i')
            ) , 
            'required' => true , 
            'order' => 1 , 
            'label' => 'name'
        ));
        
        //Method
        $this->addElement('radio', 'method', array(
            'label' => 'action' , 
            'order' => 7 , 
            'value' => 1 ,
            'multiOptions' => array(
                1 => $translate->_('createOrOverwrite') ,
                2 =>  $translate->_('createOrAppend') , 
            ),
            'order' => 2 ,
        ));
        
        //Dbf
        $this->addElement('file', 'dbf', array(
            'destination' => GT_TMP_PATH , 
            'validators' => array(
                array(
                    'Extension' , 
                    false , 
                    'dbf'
                ) , 
                array(
                    'Count' , 
                    false , 
                    1
                ) , 
                array(
                    'FilesSize' , 
                    false , 
                    array(
                        '1' , 
                        '20MB'
                    )
                )
            ) , 
            'required' => true , 
            'order' => 4 , 
            'label' => '.dbf'
        ));
        
        //Shp
        $this->addElement('file', 'shp', array(
            'destination' => GT_TMP_PATH , 
            'validators' => array(
                array(
                    'Extension' , 
                    false , 
                    'shp'
                ) , 
                array(
                    'Count' , 
                    false , 
                    1
                ) , 
                array(
                    'FilesSize' , 
                    false , 
                    array(
                        '1' , 
                        '20MB'
                    )
                )
            ) , 
            'required' => false , 
            'order' => 5 , 
            'label' => '.shp'
        ));
        
        //Shx
        $this->addElement('file', 'shx', array(
            'destination' => GT_TMP_PATH , 
            'validators' => array(
                array(
                    'Extension' , 
                    false , 
                    'shx'
                ) , 
                array(
                    'Count' , 
                    false , 
                    1
                ) , 
                array(
                    'FilesSize' , 
                    false , 
                    array(
                        '1' , 
                        '1MB'
                    )
                )
            ) , 
            'required' => false , 
            'order' => 6 , 
            'label' => '.shx'
        ));
        
        //Prj
        $gdalsrsinfoFound = false;
        if(GT_GDAL_BIN_PATH) {
            if(file_exists(GT_GDAL_BIN_PATH . DIR_SEP . 'gdalsrsinfo.exe')) {
                $gdalsrsinfoFound = true;
            }
        } else {
            $locateResult = Giscuit_Exec::exec('whereis', 'gdalsrsinfo');
            if(trim($locateResult['output']) != 'gdalsrsinfo:') {
                $gdalsrsinfoFound = true;
            }
        }
                
        if($gdalsrsinfoFound) {
            $this->addElement('file', 'prj', array(
                'destination' => GT_TMP_PATH ,
                'validators' => array(
                    array(
                        'Extension' ,
                        false ,
                        'prj'
                    ) ,
                    array(
                        'Count' ,
                        false ,
                        1
                    ) ,
                    array(
                        'FilesSize' ,
                        false ,
                            array(
                                '1' ,
                                '1MB'
                            )
                    )
                ) ,
                'required' => false ,
                'order' => 7 ,
                'label' => '.prj'
            ));
        }
        
        //
        $editingPlugin = Zend_Controller_Front::getInstance()->getPlugin('Giscuit_Controller_Plugin_Editing');
        if ($editingPlugin != false) {
            $editingPlugin->adminDatabaseForm($this);
        }
        
        //
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'id' => 'databaseSubmit' , 
            'order' => 10
        ));
        
        $this->setAttrib('enctype', 'multipart/form-data');
    
    }

    public function addSrid ($data)
    {
        //Srid
        $this->addElement('autocomplete', 'srid', array(
            'id' => 'databaseSridAc' , 
            'required' => false , 
            'order' => 8 , 
            'label' => 'specifyProjectionManually'
        ));
        
        $this->addElement('hidden', 'srid_value', array(
            'id' => 'databaseSrid' , 
        ));
    }

    public function addEncoding ($data)
    {
        //Encoding
        $this->addElement('autocomplete', 'encoding', array(
            'filters' => array(
                'StringToUpper'
            ) , 
            'id' => 'databaseEncodingAc' , 
            'required' => true , 
            'value' => 'UTF-8' , 
            'order' => 3 , 
            'label' => 'encoding'
        ));
    
    }

}