<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersAdd.php 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Add layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersAdd extends Giscuit_Form_Abstract
{
    
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        
        //
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtLayersAddForm'
        ));
        
        //Type
        $typeArr = array(
            'container' => $translate->_('container') ,
            'vector' => $translate->_('vector') 
        );
        if ($isAllowed->direct('admin')) {
            $typeArr['raster'] = $translate->_('raster');
            $typeArr['wms'] = $translate->_('wms');
            $typeArr['wfs'] = $translate->_('wfs');
            $typeArr['javascript'] = $translate->_('javascriptForDevelopers');
        }
        $this->addElement('radio', 'type', array(
            'required' => false , 
            'label' => 'type' , 
            'value' => 'container' ,
            'multiOptions' => $typeArr
        ));        
        
        //Vector
        
        //Table
        $geometryColumnsTable = new Giscuit_Db_Table_GeometryTables();
        $isAllowed = Zend_Controller_Action_HelperBroker::getStaticHelper('isAllowed');
        $usersTable = new Giscuit_Db_Table_Users();
        if (! $isAllowed->direct('admin')) {
            $userTables = (string)$usersTable->getItems(array(
                'whereId' => Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->direct()
            ))->current()->tables;
            $tables = $geometryColumnsTable->getItems(array('whereTables' => $userTables));
        } else {
            $tables = $geometryColumnsTable->getItems();
        }
        $tablesArr = array();
        foreach ($tables as $table) {
            $tablesArr[$table->f_table_name] = $table->f_table_name;
        }
        $this->addElement('select', 'table', array(
            'required' => false ,
            'label' => 'table' ,
            'multiOptions' => $tablesArr
        ));
        
        //Class
        $this->addElement('text', 'class', array(
            'filters' => array(
                'Classname'  
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        'min' => 3,
                        'max' => 30,
                    )
                ) , 
                'Classname'
            ) , 
            'required' => false , 
            'label' => 'class'
        ));
        
        //Raster
        
        //Data        
        $this->addElement('text', 'data', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                )
            ) , 
            'required' => false , 
            'label' => 'data'
        ));
        
        //Tileindex
        $this->addElement('text', 'tileindex', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                )
            ) , 
            'required' => false , 
            'label' => 'tileindex'
        ));
        
        //Tileitem
        $this->addElement('text', 'tileitem', array(
            'required' => false , 
            'label' => 'tileitem'
        ));
        
        //Wms
        
        //Uri
        $this->addElement('text', 'owsclient_uri', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    3
                ),
                array(
                    'Uri'
                )
            ) , 
            'required' => false , 
            'label' => 'uri'
        ));
        
        $this->addAuthSection();

        //Layers
        $this->addElement('select', 'owsclient_layers', array(
            'required' => false , 
            'label' => 'layersUpdatedOnParamsChange',
            'registerInArrayValidator' => false,
        ));
        
        //Layer metadata
        $this->addElement('hidden', 'owsclient_metadata', array(
            'required' => false
        ));
        
        $this->addNameSection();
        
        //Submit
        $this->addElement('submit', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'id' => 'layersSubmit'
        ));
        
    }
    
    /**
     * Add name section
     * 
     * @return void
     */
    public function addNameSection ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        $languages = Zend_Registry::get('config')->language->toArray();
        $nameFields = array();
        foreach ($languages as $key => $language) {
            $this->addElement('text', 'name_' . $key, array(
                'filters' => array(
                    'HtmlSpecialChars'
                ) ,
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    ),
                ) , 
                'required' => false , 
                'label' => $language
            ));
            $nameFields[] = 'name_' . $key;
        }
        $this->addDisplayGroup($nameFields, 'nameGroup');
        $nameGroup = $this->getDisplayGroup('nameGroup');
        $nameGroup->setLegend($translate->_('name'));
    }
    
    /**
     * Add authentication section
     * 
     * @return void
     */
    public function addAuthSection()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        //owsclient_auth_type
        $typeArr = array(
            '' => '' ,
            'any' => 'any' , 
            'basic' => 'basic' , 
            'ntlm' => 'ntlm' ,
        );
        $this->addElement('select', 'owsclient_auth_type', array(
            'label' => 'type' , 
            'multiOptions' => $typeArr
        ));
        
        //         
        $this->addElement('text', 'owsclient_auth_username', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 100
                    )
                )
            ) , 
            'required' => false , 
            'renderPassword' => true,
            'label' => 'username'
        ));
        
        //
        $this->addElement('password', 'owsclient_auth_password', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 100
                    )
                )
            ) ,
            'required' => false ,
            'renderPassword' => true,
            'label' => 'password'
        ));

        $this->addDisplayGroup(array(
            'owsclient_auth_type', 'owsclient_auth_username', 'owsclient_auth_password'
        ),'authGroup');
        $authGroup = $this->getDisplayGroup('authGroup');
        $authGroup->setLegend($translate->_('authentication'));
    }
    
    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */ 
    public function populate (array $values)
    {
        $type =  $values['type'];
        
        if($type == 'container') {
            $class = $this->getElement('class');
            $class->setRequired(false);
            $class->setValue('');
        }
        
        $result = $this->setDefaults($values);
        
        return $result;
       
    }
    
}