<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: LayersEditVector.php 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Form
 */
require_once 'Zend/Form.php';

/**
 * Edit vector layer form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @subpackage Admin
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Admin_LayersEditVector extends Giscuit_Form_Abstract
{
    /**
     * Rules table
     * 
     * @var Giscuit_Db_Table_Rules
     */
    protected $_rulesTable;

    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        //Submit
        $this->addElement('button', 'submit_button', array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'submit' , 
            'id' => 'layersSubmit'
        ));
        
        //Form view helper
        $this->addDecorators(array(
            array(
                'viewScript' , 
                array(
                    'viewScript' => 'forms' . DIR_SEP . 'layersEditVector' . GT_SCRIPT_EXT
                )
            )
        ));
    
    }

    /**
     * Add general section
     * 
     * @param integer $id Layer id
     * @return void
     */
    public function addGeneralSection ($id)
    {
               
        //Active
        $this->addElement('checkbox', 'active', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        $containers = Giscuit_Db_Table_Layers::getContainers($id);
        
        $this->addElement('select', 'parent_id', array(
            'required' => true , 
            'label' => 'container' , 
            'multiOptions' => $containers 
        ));
        
        //Update interval
        $this->addElement('text', 'owsclient_update_interval', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'updateIntervalMinutes'
        ));
                
        //Minscaledenom
        $this->addElement('text', 'minscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'minscaledenom'
        ));
        
        //Maxscaledenom
        $this->addElement('text', 'maxscaledenom', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'maxscaledenom'
        ));
        
        //Opacity
        $this->addElement('text', 'opacity', array(
            'validators' => array(
                'Digits' , 
                array(
                    'Between' , 
                    false , 
                    array(
                        0 , 
                        100
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'opacity'
        ));
        
        //Filter
        $this->addElement('text', 'filter', array(
            'required' => false , 
            'label' => 'filter'
        ));
        
        //Edit maxscale
        $this->addElement('text', 'edit_maxscale', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'edit_maxscaledenom'
        ));
        
        //Edit maxscale
        $this->addElement('text', 'snap_maxscale', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'snap_maxscaledenom'
        ));
        
        //Tolerance
        $this->addElement('text', 'tolerance', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'tolerance'
        ));
        
        //Search scale
        $this->addElement('text', 'search_scale', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'search_scaledenom'
        ));

    }

    /**
     * Add name section
     * 
     * @return void
     */
    public function addNameSection ()
    {
        $languages = Zend_Registry::get('config')->language->toArray();
        foreach ($languages as $key => $language) {
            $this->addElement('text', 'name_' . $key, array(
                'filters' => array(
                    'HtmlSpecialChars'
                ) ,
                'validators' => array(
                    array(
                        'StringLength' , 
                        false , 
                        3
                    ),
                ) , 
                'required' => false , 
                'label' => $language
            ));
        }
    }

    /**
     * Add default settings section
     * 
     * @return void
     */
    public function addSettingsSection ()
    {
        //View
        $this->addElement('checkbox', 'view', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'view' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Select
        $this->addElement('checkbox', 'select', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'information' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Label
        $this->addElement('checkbox', 'label', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'label' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Edit
        $this->addElement('checkbox', 'edit', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'edit' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    
    }
    
    /**
     * Add classes and sytles section
     * 
     * @return void
     */
    public function addClassesAndSytlesSection()
    {
        //Active
        $this->addElement('checkbox', 'classlabel_active', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'classlabel_active' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }

    /**
     * Add label section
     * 
     * @return void
     */
    public function addLabelSection ()
    {
        //Size
        $this->addElement('text', 'label_size', array(
            'validators' => array(
                'Digits' , 
                array(
                    'Between' , 
                    false , 
                    array(
                        4 , 
                        32
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'label_size'
        ));
        
        //Font
        $fontsArr = Giscuit_Map_Map::getFontList(array(
            'forForm' => true
        ));
        $this->addElement('select', 'label_font', array(
            'required' => false , 
            'label' => 'label_font' , 
            'multiOptions' => $fontsArr
        ));
        
        //Color
        $this->addElement('text', 'label_color', array(
            'validators' => array(
                'Color'
            ) , 
            'required' => true ,
            'label' => 'label_color' ,
        ));
        
        //Backgroundcolor
        $this->addElement('text', 'label_backgroundcolor', array(
            'validators' => array(
                'Color'
            ) , 
            'required' => false , 
            'label' => 'label_backgroundcolor'
        ));
        
        //Fields
        $this->addElement('multiCheckbox', 'label_fields', array(
            'required' => false , 
            'label' => 'label_fields'
        ));
        
        //Separator
        $this->addElement('text', 'label_separator', array(
            'filters' => array(
                'HtmlSpecialChars'
            ) ,
            'required' => false , 
            'label' => 'label_separator'
        ));
        
        //Buffer
        $this->addElement('text', 'label_buffer', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => true , 
            'label' => 'label_buffer'
        ));
        
        //Mindistance
        $this->addElement('text', 'label_minfeaturesize', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'label_minfeaturesize'
        ));
        
        //Angle
        $angleArr = array(
            'f' => '0°' , 
            'auto' => 'auto' , 
            'follow' => 'follow'
        ); 
        $this->addElement('select', 'label_angle', array(
            'required' => true , 
            'label' => 'label_angle' , 
            'multiOptions' => $angleArr
        ));
                
        //Position
        $positionArr = array(
            MS_UL => 'posUl' , 
            MS_LR => 'posLr' , 
            MS_UR => 'posUr' , 
            MS_LL => 'posLl' , 
            MS_CR => 'posCr' , 
            MS_CL => 'posCl' , 
            MS_UC => 'posUc' , 
            MS_LC => 'posLc' , 
            MS_CC => 'posCc' , 
            MS_AUTO => 'auto'
        );
        $this->addElement('select', 'label_position', array(
            'required' => false , 
            'label' => 'label_position' , 
            'multiOptions' => $positionArr
        ));
        
        //Repeat distance
        $this->addElement('text', 'label_repeatdistance', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'label_repeatdistance'
        ));
        
        //Align
        $alignArr = array(
            MS_ALIGN_LEFT => 'left' , 
            MS_ALIGN_CENTER => 'center' , 
            MS_ALIGN_RIGHT => 'right' , 
        );
        $this->addElement('select', 'label_align', array(
            'required' => false , 
            'label' => 'label_align' , 
            'multiOptions' => $alignArr
        ));
        
        //Max length
        $this->addElement('text', 'label_maxlength', array(
            'validators' => array(
                'Digits'
            ) , 
            'required' => false , 
            'label' => 'label_maxlength'
        ));
        
        //Priority
        $this->addElement('text', 'label_priority', array(
            'validators' => array(
                'Digits' ,
                array(
                    'Between' , 
                    false , 
                    array(
                        1 , 
                        10
                    )
                )
            ) , 
            'required' => false , 
            'label' => 'label_priority'
        ));
        
        //Wrap
        $this->addElement('text', 'label_wrap', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    1
                )
            ) , 
            'filters' => array(
                'HtmlSpecialChars'
            ) ,
            'required' => false , 
            'label' => 'label_wrap'
        ));
    }

    /**
     * Add permissions section
     * 
     * @return void
     */
    public function addPermissionsSection ()
    {
        //Resource
        $this->_rulesTable = new Giscuit_Db_Table_Rules();
        $resources = $this->_rulesTable->getItems(array(
            'getResourceList' => 'layer'
        ));
        $resources = $resources->toArray();
        $resourcesArr = array(
            '' => ''
        );
        foreach ($resources as $resource) {
            $resourcesArr[$resource['resource']] = $resource['resource'];
        }
        $this->addElement('select', 'resource', array(
            'required' => false , 
            'label' => 'resource' , 
            'attribs' => array(
                'id' => 'menuResource'
            ) , 
            'multiOptions' => $resourcesArr
        ));
        
        //View privilege
        $this->addElement('select', 'priv_view', array(
            'required' => false , 
            'label' => 'priv_view' , 
            'attribs' => array(
                'id' => 'menuViewPrivilege'
            )
        ));
        
        //Editgeom privilege
        $this->addElement('select', 'priv_editgeom', array(
            'required' => false , 
            'label' => 'priv_editgeom' , 
            'attribs' => array(
                'id' => 'menuEditgeomPrivilege'
            )
        ));
        
        //Editattr privilege
        $this->addElement('select', 'priv_editattr', array(
            'required' => false , 
            'label' => 'priv_editattr' , 
            'attribs' => array(
                'id' => 'menuEditattrPrivilege'
            )
        ));
        
        //Ows privilege
        $this->addElement('select', 'priv_ows', array(
            'required' => false , 
            'label' => 'priv_ows' , 
            'attribs' => array(
                'id' => 'menuOwsPrivilege'
            )
        ));
    }

    /**
     * Add tables and fields section
     * 
     * @return void
     */
    public function addTableAndFieldsSection ()
    {
        //Default field
        $this->addElement('select', 'default_field', array(
            'required' => false , 
            'label' => 'default_field' , 
        ));

    }
    
    /**
     * Add editing rules section
     * 
     * @return void
     */
    public function addEditSection()
    {
        $this->addElement('textarea', 'form' , array(
            'filters' => array() , 
            'validators' => array(
                array(
                    'Xml' 
                )
            ) , 
            'required' => false ,
            'rows' => 20 , 
            'label' => 'form' , 
        ));
    }
    
    /**
     * Add wms section
     * 
     * @return void
     */
    public function addWmsSection()
    {
        //Private
        $this->addElement('checkbox', 'wms_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wms_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add wfs section
     * 
     * @return void
     */
    public function addWfsSection()
    {
        //Private
        $this->addElement('checkbox', 'wfs_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wfs_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
    }
    
    /**
     * Add wcs section
     * 
     * @return void
     */
    public function addWcsSection()
    {
        //Private
        $this->addElement('checkbox', 'wcs_private', array(
            'validators' => $this->checkboxValidator , 
            'required' => true , 
            'label' => 'publishPrivately' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Public
        $this->addElement('checkbox', 'wcs_public', array(
            'validators' => $this->checkboxValidator ,
            'required' => true , 
            'label' => 'publishPublicly' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Formats
        $formats = array_merge(array('' => ''), Giscuit_Map_Map::getOutputformatsList());
        $this->addElement('multiselect', 'wcs_formats', array(
            'required' => false , 
            'label' => 'outputFormats' ,
            'multiOptions' => $formats
        ));
    }
    
    /**
     * Add publish section
     * 
     * @return void
     */
    public function addPublishSection($className)
    {     
         //Destination
        $destinationArr = array(
            'overlay' => 'overlay' 
        );
        $baseTable = new Giscuit_Db_Table_Base();
        $baseRows = $baseTable->getItems();
        foreach($baseRows as $row) {
            $destinationArr[$row->id] = $row->name;
        }
        
        $this->addElement('select', 'destination', array(
            'required' => false , 
            'label' => 'publish_destination' , 
            'isArray' => true , 
            'multiOptions' => $destinationArr
        ));
        
        
        //Wms format
        $imageFormatArr = array(
            'png' => 'png' , 
            'png24' => 'png24' , 
        );
        $this->addElement('select', 'format', array(
            'required' => false , 
            'label' => 'overlay_format' , 
            'multiOptions' => $imageFormatArr ,
            'value' => 'png24'
        ));
        
        //Selected
        $this->addElement('checkbox', 'selected', array(
            'validators' => $this->checkboxValidator ,
            'value' => 'f' , 
            'required' => true , 
            'label' => 'publish_selected' , 
            'checkedValue' => 't' ,
            'uncheckedValue' => 'f'
        ));
        
        //Clear cache
        $clearCacheDisabled = array();
        if(! file_exists(GT_PUBLIC_PATH . DIR_SEP . 'tiles' . DIR_SEP . $className)) {
            $clearCacheDisabled['disabled'] = 'disabled';    
        }
        $this->addElement('button', 'clear_cache', array_merge($clearCacheDisabled, array(
            'required' => false , 
            'ignore' => true , 
            'label' => 'clearCache' , 
            'id' => 'clearCache' ,
        )));
        
    }
    
    /**
     * Add authentication section
     * 
     * @return void
     */
    public function addAuthSection()
    {

        //owsclient_auth_type
        $typeArr = array(
            '' => '' ,
            'any' => 'any' , 
            'basic' => 'basic' , 
            'ntlm' => 'ntlm' ,
        );
        $this->addElement('select', 'owsclient_auth_type', array(
            'label' => 'format' , 
            'multiOptions' => $typeArr
        ));
        
        //         
        $this->addElement('text', 'owsclient_auth_username', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 100
                    )
                )
            ) , 
            'required' => false , 
            'label' => 'username'
        ));
        
        //
        $this->addElement('password', 'owsclient_auth_password', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array (
                        'max' => 500
                    )
                )
            ) ,
            'required' => false ,
            'renderPassword' => true,
            'label' => 'password'
        ));
        
    }

    /**
     * Populate form
     *
     * Proxies to {@link setDefaults()}
     *
     * @param  array $values
     * @return Zend_Form
     */
    public function populate (array $values)
    {
        $translate = Zend_Registry::get('Zend_Translate');

        $mapOptions = array(
            'orderDirection' => 'desc' ,
            'withFields' => true 
        );
        
        $table = Giscuit_Db_Table_Layers::getLayerTableById($values['id'], $mapOptions);
        $settings = $table->getSettings();
        
        $fields = $table->getPublicFields();
        $fieldsArr = array();
        foreach ($fields as $field) {
            $fieldsArr[$field['name']] = $translate->_($field['name']);
        }
        
        $defaultLabelFields = $this->getElement('label_fields');
        $defaultLabelFields->setMultiOptions($fieldsArr);
                
        //Privledges
        if (! empty($values['resource'])) {
            $privileges = $this->_rulesTable->getItems(array(
                'getPrivilegeList' => $values['resource']
            ));
            $privilegesArr = array(
                '' => ''
            );
            foreach ($privileges as $privilege) {
                $privilegesArr[$privilege['privilege']] = $privilege['privilege'];
            }
            
            $this->getElement('priv_view')->setMultiOptions($privilegesArr);
            if($this->getElement('priv_editgeom')) {
                $this->getElement('priv_editgeom')->setMultiOptions($privilegesArr);
            }
            if($this->getElement('priv_editattr')) {
                $this->getElement('priv_editattr')->setMultiOptions($privilegesArr);
            }
            $this->getElement('priv_ows')->setMultiOptions($privilegesArr);
        }
        
        //Default field
        $defaultFieldElement = $this->getElement('default_field');
        $defaultFieldElement->setMultiOptions($fieldsArr);
        
        //Routing
        if ($this->getDisplayGroup('routing') != NULL) {
            if (! isset($values['routing_algorithm']) || ! $values['routing_algorithm']) {
                $values['routing_algorithm'] = 2;
            } else {
                $values['routing_algorithm'] = explode(',', $values['routing_algorithm']);
            }
        }
        
        //Form
        $formElement = $this->getElement('form');
        if($formElement) {
            $formValue = $formElement->getValue();
            if(empty($formValue)) {
                $configPath = GT_LIBRARY_PATH . DIR_SEP . 'Giscuit' . DIR_SEP . 'Form' . DIR_SEP . 'Layer' . DIR_SEP . $settings['class'] . '.xml';
                $formContent = file_get_contents($configPath);
                $formElement->setValue($formContent);
            }
        }
        
        $result = $this->setDefaults($values);
        
        
        return $result;
    }
}