<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: Signup.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Giscuit_Form_Abstract
 */
require_once 'Giscuit/Form/Abstract.php';

/**
 * User signup form class
 * 
 * @category   Giscuit
 * @package    Giscuit_Form
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Form_Signup extends Giscuit_Form_Abstract
{
    /**
     * Initialize
     *
     * @return void
     */
    public function init ()
    {
        $translate = Zend_Registry::get('Zend_Translate');
        
        //
        $this->setMethod('post');
        $this->addAttribs(array(
            'id' => 'gtAuthSignupForm'
        ));
        
        //Account
        $this->addElement('text', 'account', array(
            'filters' => array(
                'StringTrim',
                'StringtoLower'
            ) , 
            'validators' => array(
                array(
                    'EmailAddress'
                ) , 
                array(
                    'IdAvailable'
                )
            )
             , 
            'required' => true , 
            'label' => 'accountAndEmail'
        ));
        
        //Password
        $this->addElement('password', 'password', array(
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        6 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'password'
        ));
        
        //Confirm password
        $this->addElement('password', 'confirm_password', array(
            'validators' => array(
                array(
                    'ConfirmPassword' , 
                    false , 
                    array(
                        'password'
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'confirm_password'
        ));
        
        $this->addDisplayGroup(array(
            'account' , 
            'password' , 
            'confirm_password' , 
        ), 'accountInfo');
        
        $accountInfo = $this->getDisplayGroup('accountInfo');
        $accountInfo->setLegend($translate->_('accountInfo'));
        
        //First name
        $this->addElement('text', 'fname', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        3 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'fname'
        ));
        
        //Last name
        $this->addElement('text', 'lname', array(
            'filters' => array(
                'StringTrim',
                'HtmlSpecialChars',
                array(
                    'Callback',
                    'ucwords'
                )
            ) , 
            'validators' => array(
                array(
                    'StringLength' , 
                    false , 
                    array(
                        3 , 
                        20
                    )
                )
            ) , 
            'required' => true , 
            'label' => 'lname'
        ));
        
        //Time zone
        $timeZoneArr = array_combine(DateTimeZone::listIdentifiers(), DateTimeZone::listIdentifiers());
        $config = Zend_Registry::get('config');
        $this->addElement('select', 'timezone', array(
            'required' => true , 
            'label' => 'Timezone' , 
            'multiOptions' => $timeZoneArr ,
            'value' => $config->timeZone
        ));
        
        
        $this->addDisplayGroup(array(
            'fname' , 
            'lname' ,
            'timezone'
        ), 'personal');
        $personal = $this->getDisplayGroup('personal');
        $personal->setLegend($translate->_('personalInfo'));

        //License agreement
        $this->addElement('checkbox', 'agreement', array(
            'validators' => array(
                array(
                    'Identical' , 
                    false , 
                    array(
                        't'
                    )
                )
            ) ,
            'required' => true , 
            'label' => vsprintf($translate->_('iAgreeTerms'), 'signupagreement') , 
            'checkedValue' => 't' , 
            'uncheckedValue' => 'f'
        ));
        $this->getElement('agreement')->getDecorator('label')->setOption('escape', false);
        $this->getElement('agreement')->getValidator('Identical')->setMessage($translate->_('youNotAgreeTerms'), 'notSame');
        
        
        $this->addDisplayGroup(array(
            'agreement' , 
        ), 'signupAgreement');
        $personal = $this->getDisplayGroup('signupAgreement');
        $personal->setLegend($translate->_('signupAgreement'));
        
        //Submit
        $signup = $this->addElement('button', 'submit_button', array(
            'attribs' => array(
                'id' => 'gtAuthSignupSubmit'
            ) , 
            'required' => false , 
            'ignore' => true , 
            'label' => 'signup'
        ));
        
    }
}