<?php
/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this application in the file LICENSE.txt.
 *
 * @version    $Id: GeometryType.php 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */

/**
 * @see Zend_Validate_Abstract
 */
require_once 'Zend/Validate/Abstract.php';

/**
 * WKT geometry type validator
 * 
 * @category   Giscuit
 * @package    Giscuit_Validate
 * @copyright  Copyright (c) 2012 VEC
 */
class Giscuit_Validate_GeometryType extends Zend_Validate_Abstract
{

    const INVALID_GEOMETRY_TYPE = 'invalidGeometryType';

    /**
     * Validation failure message template definitions
     *
     * @var array
     */
    protected $_messageTemplates = array(
        self::INVALID_GEOMETRY_TYPE => "Invalid geometry type"
    );
    
    /**
     * Type to compare with
     * 
     * @var Zend_Validate_Abstract
     */
    protected $_type;

    /**
     * Sets validator options
     *
     * @param Zend_Validate_Abstract $type
     * @return void
     */
    public function __construct ($type = null)
    {
        $this->setType($type);
    }

    /**
     * Get the type
     *
     * @return Zend_Validate_Abstract
     */
    public function getType ()
    {
        return $this->_type;
    }

    /**
     * Set the type
     * 
     * @param mixed $type
     * @return Zend_Validate_Abstract
     */
    public function setType ($type)
    {
        $this->_type = $type;
        return $this;
    }

    /**
     * Defined by Zend_Validate_Interface
     * 
     * Returns true if and only if geometry matches the specified type
     * 
     * @param string $value
     * @return boolean
     */
    public function isValid ($value)
    {

        $config = Zend_Registry::get('config');
        $db = Zend_Registry::get('db');
                
        $value = htmlspecialchars($value, ENT_QUOTES);
        
        $theGeom = "st_geomfromtext(:the_geom, {$config->map->srid})";
                
        $stmt = $db->prepare("SELECT st_geometrytype($theGeom) as result");
        $stmt->bindParam('the_geom', $value, PDO::PARAM_STR);
        
        try {
            $stmt->execute();
            $result = $stmt->fetch();
            
            if (stristr($result->result, $this->getType())) {
                return true;
            } else {
                $this->_error(self::INVALID_GEOMETRY_TYPE);
                return false;
            }
        } catch (Zend_Db_Exception $e) {
            $this->_error(self::INVALID_GEOMETRY_TYPE);
            return false;
        }
    
    }
}