/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Editattr.js 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Editattr
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Editattr = OpenLayers.Class(Giscuit.Control, {

    feature: null,

    panel: null,

    submitButton: null,
    
    cancelButton: null,
    
    versionsSelect: null,
    
    dateInstances: null,
    
    initialize: function (feature, version) {
        if(feature != null) {
            this.feature = feature;
        }
        
        var getCallback = {
            success: this.getSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        if(this.panel == null) {
            gt.accord.addPanel({
                label: gt.translate._('editAttributes') ,
                content: this.getLoadingImage()
            });
            this.panel = gt.accord.getPanel(gt.accord.getPanels().length - 1);
            this.addHelpButton('http://docs.giscuit.com/En/tutorial_refguide.user.desktop.edit.pkg.html#online');
        } else {
            gt.accord.setBody(this.getPanelIndex(), this.getLoadingImage());
        }   
        gt.accord.openPanel(this.getPanelIndex());

        var idStm = '';
        if(this.feature.attributes.gid != null) {
            idStm = "/id/" + this.feature.attributes.gid;
        } 
                
        var request = YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom/get" + idStm + "/version/" + version, getCallback);
    },

    getSuccess: function (o) {
        
        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        } 
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        var setBodySuccess = gt.accord.setBody(this.getPanelIndex(), responseText.form);
        if(! setBodySuccess) {
            return;
        }
        
        if(responseText.geometry != undefined) {
            //Unselect
            gt.control.editgeom.controls.editControl.unselectFeature(this.feature);
            gt.control.editgeom.controls.editControl.selectControl.unselectAll();
            
            //Change geometry
            gt.layer.edit.removeFeatures(this.feature, true);
            this.feature.geometry.destroy();
            this.feature.geometry = new OpenLayers.Geometry.fromWKT(responseText.geometry);
            gt.layer.edit.addFeatures(this.feature, true);
            
            //Select
            gt.control.editgeom.controls.editControl.selectFeature(this.feature);
            gt.control.editgeom.controls.editControl.selectControl.select(this.feature);
            
            this.feature.state = OpenLayers.State.UPDATE;
        }
        
        this.initInput();
        
        //Trigger form render event
        gt.control.editgeom.events.triggerEvent("formrender", YAHOO.util.Dom.get('gtEditattrForm'));
        
    },

    submitSuccess: function (o) {

        try {
            var responseText = YAHOO.lang.JSON.parse(o.responseText);
        } catch(e) {
            Giscuit.Util.parseFailure(o, e);
            return;
        } 
        
        if (responseText.replyCode != 200) {
            Giscuit.Util.invalidReplyCode(responseText);
            return;
        }
        
        if (! responseText.success) {
            if (responseText.errors) {
                for (var error in responseText.errors) {
                    Giscuit.Util.message(responseText.errors[error]);
                }
                if(gt.control.editgeom.controls.editControl.active) {
                    gt.accord.setBody(this.getPanelIndex(), responseText.form);
                    this.initInput();
                } else {
                    gt.control.editgeom.controls.editControl.selectControl.unselectAll();
                    gt.control.editgeom.clearInsertedFeatures();
                    this.destroy();
                }
            } else {
                gt.accord.setBody(this.getPanelIndex(), responseText.form);
                this.initInput();
            }
            
        } else {
            //Submit
            
            //Destroy original
            if(this.feature.originalGeometry != undefined) {
                this.feature.originalGeometry.destroy();
                this.feature.originalGeometry = undefined;
            }
            this.feature.state = null;
            
            gt.control.editgeom.controls.editControl.selectControl.unselectAll();
                            
            if(responseText.gid != undefined) {
                this.feature.attributes.gid = responseText.gid;
            }

            gt.control.editgeom.backActivate();

            //Update attributes
            var values = YAHOO.util.Connect.setForm('gtEditattrForm').split('&');
            for(var i=0; i < values.length; i++) {
                var key = decodeURIComponent(values[i].split('=')[0]);
                var value = decodeURIComponent(values[i].split('=')[1]);
                
                this.feature.attributes[key] = value;
            }

            //Trigger add/edit event with feature attributes
            var data = {
                values: this.feature.attributes,
                feature: this.feature
            };
            
            if (typeof (responseText.gid) != 'undefined') {
                gt.control.editgeom.events.triggerEvent("addfeature", data);
            } else {
                gt.control.editgeom.events.triggerEvent("editfeature", data);
            }
            
            //FIXME:
            gt.layer.edit.removeFeatures(this.feature, true);
            gt.layer.edit.addFeatures(this.feature, true);
                            
            this.destroy();
        }

    },

    submit: function () {
        this.submitButton.set('disabled', true);

        var submitCallback = {
            upload: this.submitSuccess,
            timeout: this.callbackTimeout,
            failure: this.callbackFailure,
            scope: this
        };
        
        var postData = '';
        if(this.feature.geometry != null) {
            postData = 'geometry=' + this.feature.geometry;
        }
        
        var stm = '';
        if(this.feature.attributes.gid != null) {
            stm = '/edit/id/' + this.feature.attributes.gid;
        } else {
            stm = '/add';
        }
        
        YAHOO.util.Connect.setForm('gtEditattrForm', true);
        YAHOO.util.Connect.asyncRequest('POST', gt.config.httpPath + "/editgeom" + stm, submitCallback, postData);

    },

    initInput: function () {
        var dom = YAHOO.util.Dom;
        
        //Submit
        this.submitButton = new YAHOO.widget.Button('gtEditattrSubmit');
        this.submitButton.on('click', this.submit, null, this);
        
        //Cancel
        this.cancelButton = new YAHOO.widget.Button('gtEditattrCancel');
        this.cancelButton.on('click', function() {
            gt.control.editgeom.controls.editControl.selectControl.unselectAll();
            gt.control.editgeom.clearInsertedFeatures();
            this.destroy();
        }, null, this);
        
        //
        var versionSelect = dom.get('gtEditattrVersionsSelect');
        if(versionSelect) {
            this.versionsSelect = new YAHOO.widget.Button("gtEditattrVersionsButton", { 
                label: versionSelect[versionSelect.selectedIndex].text ,
                type: "menu",  
                menu: "gtEditattrVersionsSelect" 
            }); 
            this.versionsSelect.getMenu().subscribe("click", function(evt, data) {
                this.changeVersion(data[1].value);
            }, null, this);
        }

        //
        var dateElements = dom.getElementsByClassName('gt-field-date');
        if (dateElements.length != 0) {
            for (var i = 0; i < dateElements.length; i++) {
                this.addCalendar(dateElements[i]);
            }
        }
        
        //Toggle
        var nodes = YAHOO.util.Selector.query('#gtEditattrPH fieldset');
        for(var i = 0; i < nodes.length; i++) {
            var legend = nodes[i].getElementsByTagName('legend')[0];

            //Hide label
            nodes[i].parentNode.previousSibling.style.display = 'none';

            //Status
            var status = 'off'
            if(dom.getElementsByClassName('errors', 'ul', nodes[i]).length) {
                var status = 'on'
            }
            dom.addClass(nodes[i], status);
            
            //Add span
            var toggleSpan = document.createElement('span');
            legend.appendChild(toggleSpan);
        }
        //Toggle events
        YAHOO.util.Event.delegate('gtEditattrPH', "click", function(evt, matchedEl, container) {
            var fieldset = matchedEl.parentNode;
            
            //
            var status = 'on';
            if(dom.hasClass(fieldset, 'on')) {
                status = 'off';
            } 
            
            //Reset
            var nodes = YAHOO.util.Selector.query('#gtEditattrPH fieldset');
            for(var i = 0; i < nodes.length; i++) {
                dom.removeClass(nodes[i], 'on');
                dom.addClass(nodes[i], 'off');
            }

            //Set
            dom.addClass(fieldset, status);
                        
        }, "legend"); 
        
        //
        var heightCoef = 30;
        dom.get('gtEditattrPH').style.maxHeight = gt.config.accordPanelMaxHeight - heightCoef + 'px';
    },
    
    changeVersion: function(id) {
        this.initialize(null, id);
    },
    
    addCalendar: function(dateElement) {
        var Event = YAHOO.util.Event,
            Dom = YAHOO.util.Dom;
        var oCalendarMenu;

        var onButtonClick = function (e, dateInstances) {
            var oCalendar = new YAHOO.widget.Calendar(dateElement.id + "-buttoncalendar", oCalendarMenu.body.id);
            
            oCalendar.beforeRenderEvent.subscribe(function (p_sType, p_aArgs) {
                var date = Giscuit.Util.parseDate(dateElement.value);
                oCalendar.select(date);
            });
            
            oCalendar.render();
            
            oCalendar.selectEvent.subscribe(function (p_sType, p_aArgs) {
                var aDate;
                if (p_aArgs) {
                    aDate = p_aArgs[0][0];
                    dateElement.value = aDate[0] + '-' + aDate[1] + '-' + aDate[2];
                }
                oCalendarMenu.hide();
            });
            
            dateInstances.push(oCalendar);
            
            Event.on(oCalendarMenu.element, "keydown", function (p_oEvent) {
                if (Event.getCharCode(p_oEvent) === 27) {
                    oCalendarMenu.hide();
                    this.focus();
                }
            }, null, this);
            
            oCalendarMenu.align();
            
            this.unsubscribe("click", onButtonClick);

        };
            
        oCalendarMenu = new YAHOO.widget.Overlay(dateElement.id + "-calendarmenu", { visible: false });
        
        var oButton = new YAHOO.widget.Button({ 
            type: "menu", 
            id: dateElement.id + "-calendarpicker", 
            menu: oCalendarMenu, 
            container: dateElement.id + "-element"
        });
        
        oButton.get("element").firstChild.firstChild.style.background = 'url(' + gt.config.httpImagePath + '/calendar.gif) center center no-repeat';
        oButton.get("element").style.marginBottom = '-4px';
                
        oButton.on("appendTo", function () {
            oCalendarMenu.setBody("&#32;");
            oCalendarMenu.body.id = dateElement.id + "-calendarcontainer";
        });
        
        if (this.dateInstances == null) {
            this.dateInstances = new Array(oCalendarMenu, oButton);
        } else {
            this.dateInstances.push(oCalendarMenu, oButton);
        }
        
        oButton.on("click", onButtonClick, this.dateInstances);
    },
    
    destroy: function() {
        if(this.submitButton != null) {
            this.submitButton.destroy();
            this.submitButton = null;
        }

        if(this.cancelButton != null) {
            this.cancelButton.destroy();
            this.cancelButton = null;
        }

        if(this.versionsSelect != null) {
            this.versionsSelect.destroy();
            this.versionsSelect = null;
        }

        if(this.panel) {
            gt.accord.removePanel(this.getPanelIndex());
        }
        this.panel = null;

        if(this.dateInstances != null) {
            for (var i = this.dateInstances.length - 1; i >= 0; i--) {
                if (this.dateInstances[i].cfg == null) {
                    this.dateInstances[i] = null;
                } else {
                    this.dateInstances[i].destroy();
                }
            }
            this.dateInstances = null;
        }
        
        OpenLayers.Control.prototype.destroy.call(this);
    },
    
    CLASS_NAME: "Giscuit.Control.Editattr"
});
