/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Geocode.js 2644 2012-05-07 12:25:16Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        if (gt.config.geocode != '') {
            gt.control.geocode = new Giscuit.Control.Geocode();
        }
    }
});

/**
 * Class: Giscuit.Control.Geocode
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Geocode = OpenLayers.Class(Giscuit.Control, {

    ds: null,
    
    ac: null,
    
    selected: null,
    
    scale: null,
    
    queryDelay: 0.5,
    
    initialize: function (container) {
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        
        var extent = gt.map.getMaxExtent().clone();
        var extentParam = '';
        if (gt.config.geocode == 'google') {
            extent.transform(gt.map.getProjectionObject(), wgsProj);
            extentParam = "extent/" + extent.toBBOX();
        }
        
        var config = {
            minQueryLength: 0, 
            animVert: 0 , 
            resultTypeList: false,
            useIFrame: true,
            queryDelay: this.queryDelay
        };
    
        this.ds = new YAHOO.util.DataSource(gt.config.httpToolPath + "/geocode/index/" + extentParam);
        this.ds.responseType = YAHOO.util.DataSource.TYPE_JSON;
        this.ds.responseSchema = {
            resultsList: "records",
            fields: [
                 {key: "text"},
                 {key: "viewport"},
                 {key: "the_geom"}
            ]
        };
        this.ds.subscribe('responseEvent', function(req) {
            var result = YAHOO.lang.JSON.parse(req.response.responseText);
            this.scale = parseFloat(result.scale);
        }, null, this);
        
        this.ac = new YAHOO.widget.AutoComplete('gtGeocodeInput', 'gtGeocodeContainer', this.ds, config);
        this.ac.formatResult = function(oResultData, sQuery, sResultMatch) {
             return oResultData.text;
        };
        this.ac.itemSelectEvent.subscribe(function(sType, aArgs) { 
            aArgs[0].getInputEl().value = aArgs[2].text;
            this.moveTo(aArgs[2]);    
        }, null, this);
        
        var input = this.ac.getInputEl();
        this.ac.dataRequestEvent.subscribe(function(sType, aArgs) {
            input.style.backgroundImage = "url(" + gt.config.httpImagePath + '/spinner.gif' + ")";
        }, null, this);
        this.ac.dataReturnEvent.subscribe(function(sType, aArgs) {
            input.style.backgroundImage = "url(" + gt.config.httpImagePath + '/magnify.png' + ")";
        }, null, this);
        
    },

    activate: function() {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        return activated;
    },
    
    deactivate: function() {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
        
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        return deactivated;
    },
    
    moveTo: function(data) {
        var wgsProj = new OpenLayers.Projection("EPSG:4326");
        var geometry = OpenLayers.Geometry.fromWKT(data.the_geom);

        if (gt.config.geocode == 'google') {
            geometry.transform(wgsProj, gt.map.getProjectionObject());
        }
        
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        }
        
        this.selected = new OpenLayers.Feature.Vector(geometry);
        
        gt.layer.vector.addFeatures([this.selected]);
        
        //Zoom to viewport
        if (gt.config.geocode == 'google') {
            var viewport = new OpenLayers.Bounds(data.viewport[0], data.viewport[1], data.viewport[2], data.viewport[3]);
            viewport.transform(wgsProj, gt.map.getProjectionObject()); 
                    
            gt.map.zoomToExtent(viewport);
        } else {
            gt.map.zoomToExtent(geometry.getBounds());
            
            if(this.scale) {
                gt.map.zoomToScale(this.scale);
            }
        }
    },
    
    CLASS_NAME: "Giscuit.Control.Geocode"
});