/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Publiclayers.js 3110 2012-11-08 08:02:59Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Class: Giscuit.Control.Publiclayers
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Control.Publiclayers = OpenLayers.Class(Giscuit.Control, {

    container: null,
    
    tree: null,
    
    nodes: {},
    
    records: [],
    
    tooltip: null,
    
    initialize: function (options) {
        //Deprecated in 2.12
        //this.EVENT_TYPES.push('changelayer');
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
        
        this.container = options.container;
        
        this.tree = new YAHOO.widget.TreeView(this.container);
        this.tree.subscribe('highlightEvent', this.highlight, null, this);
        this.tree.subscribe('clickEvent', this.tree.onEventToggleHighlight);

    },
    
    render: function() {
        this.tree.setNodesProperty('propagateHighlightUp', true); 
        this.tree.setNodesProperty('propagateHighlightDown', true);
        
        //Remove empty containers
        var removed = null;
        while(removed != false) {
            removed = false;
            for(var obj in this.nodes) {
                var node = this.nodes[obj];
                if(node.children.length == 0) {
                    if(! node.data.layerClass) {
                        this.tree.removeNode(node);
                        delete this.nodes[obj];
                        removed = true;
                    }
                }
            }
        }
        
        if(this.nodes['base'] != undefined) {
            this.nodes['base'].insertAfter(this.tree.getRoot().children[this.tree.getRoot().children.length - 1]);
        }
                
        this.tree.render();
        
        this.add();

        this.setVisibility();
        
        this.setAttributions();

        var nodes = this.tree.getNodesBy(function(node) {
            if(node.data.layerClass) {
                return true;
            } else {
                return false;
            }
        });
        
        var metadataEls = [];
        if(nodes) { 
            for (var i = 0; i < nodes.length; i++) {
               metadataEls.push(nodes[i].labelElId);
            }
            metadataEls.push('gtMetadataTooltip');
        }

        //Create tooltip for tree
        if(this.tooltip != null) {
            this.tooltip.destroy();
        }
        this.tooltip = new YAHOO.widget.Tooltip("gtMetadataTooltip", {
            context: metadataEls ,
            hidedelay: 500 ,
            showdelay: 1500 ,
            autodismissdelay: 1000 * 60
        });
        
        //Set the tooltip on top of the tree
        this.tooltip.bringToTop();

        this.tooltip.doShow = Giscuit.Util.tooltipDoShow;
       
        //Set text for the tooltip before display
        this.tooltip.contextTriggerEvent.subscribe(function(type, args, tree) {
            var text = '';
            var data = tree.getNodeByElement(args[0]).data;
            var metadata = data.metadata;
            
            if (metadata instanceof(Array) == false) {
                var metadata = [metadata];
            }
            var metadataLength = metadata.length;
            
            //
            for (var i = 0; i < metadataLength; i++) {
                text += '<ul>' + '<b>' + metadata[i].ows_title + '</b>';
                
                //maxscaledenom
                if(data.maxscaledenom) {
                    if(metadata[i].maxscaledenom) {
                        if(data.maxscaledenom < metadata[i].maxscaledenom) {
                            metadata[i].maxscaledenom = data.maxscaledenom;
                        }
                    } else {
                        metadata[i] = YAHOO.lang.merge({maxscaledenom: data.maxscaledenom}, metadata[i]);
                    }
                }
                
                //minscaledenom
                if(data.minscaledenom) {
                    if(metadata[i].minscaledenom) {
                        if(data.minscaledenom > metadata[i].minscaledenom) {
                            metadata[i].minscaledenom = data.minscaledenom;
                        }
                    } else {
                        metadata[i] = YAHOO.lang.merge({minscaledenom: data.minscaledenom}, metadata[i]);
                    }
                }
                    
                for (var key in metadata[i]) {
                    if (metadata[i].hasOwnProperty(key) && key != 'ows_title') {
                        text += '<li>' + gt.translate._(key) + ': ' + metadata[i][key] + '</li>';
                    }
                }
                text += '</ul>';
            }

            this.cfg.setProperty("text", text);
            
        }, this.tree);
        
        gt.map.events.register("zoomend", this, this.styleLayerNameOnZoom);
            
    },
    
    styleLayerNameOnZoom: function() {
        var scale = gt.map.getScale();
        
        for(var obj in this.nodes) {
            var node = this.nodes[obj];

            if(! node.data.layerClass) {
                continue;
            }
                
            var layer = gt.map.getLayersByName(node.data.layerClass)[0];
            var label = YAHOO.util.Dom.get(node.labelElId);
            
            if(! layer) {
                return;
            }

            if(layer.minscaledenom || layer.maxscaledenom) { 
                label.style.color = '#000000';
            }
                
            if(node.highlightState == 1 &&
                (layer.minscaledenom && layer.minscaledenom > scale ||
                layer.maxscaledenom && layer.maxscaledenom < scale)) {
                    
                  label.style.color = '#DDDDDD';
            } 
        }
    },
    
    setVisibility: function() {
        //Highlight selected
        var hash = Giscuit.Util.getHash();
        
        //Load from hash
        if(hash.layers != undefined && hash.layers != '') {
            var highlighted = hash.layers.split(',');
            
            for(var obj in this.nodes) {
                var node = this.nodes[obj];
                
                for(var i=0; i < highlighted.length; i++) {
                    if(highlighted[i] == node.data.id) {
                        var layer = gt.map.getLayersByName(node.data.layerClass)[0];
                        node.highlight(true);
                        layer.setVisibility(true);
                    }
                }
            }
            
        //Use defaults
        } else {
            for(var i in this.nodes) {
                var node = this.nodes[i];
                if(node.data.layerClass) {
                    if(node.data.selected == 't') {
                        var layer = gt.map.getLayersByName(node.data.layerClass)[0];
                        node.highlight(true);
                        layer.setVisibility(true);
                    }
                    
                }
            }
        }
    },
        
    highlight: function(currentNode) {
        var highlighted = [];
        
        var currentNodeState = currentNode.highlightState;
        
        //Base layers
        if (currentNode.parent == this.nodes['base']) {
            var baseLayers = this.nodes['base'].children;
            for(var i=0; i < baseLayers.length; i++) {
                baseLayers[i].unhighlight(true);
            }
            
            if(currentNodeState == 1) {
                currentNode.highlight(true);
            } 
        }
        
        //Overlays
        for(var obj in this.nodes) {
            var node = this.nodes[obj];
            
            if(node.data.layerClass) {
                if(gt.map.getLayersByName(node.data.layerClass).length) {
                    var layer = gt.map.getLayersByName(node.data.layerClass)[0];
                } else {
                    debug.warn('Layer ' + node.data.layerClass + ' not found in the haystack')
                }
                
                if(node.highlightState == 0) {
                    if(layer.getVisibility() == true) {
                        debug.info('Layer ' + node.data.layerClass + ' has visibility disabled');
                        layer.setVisibility(false);
                    }
                } else {
                    if(layer.getVisibility() == false) {
                        debug.info('Layer ' + node.data.layerClass + ' has visibility enabled');
                        layer.setVisibility(true);
                    }
                }
                
                if(node.highlightState == 1) {
                    highlighted.push(node.data.id);
                }
            }
   
        }
               
        var hash = Giscuit.Util.getHash();
        hash.layers = highlighted.join(',');
        Giscuit.Util.setHash(hash);

        //
        this.styleLayerNameOnZoom();
        
        //Attributions
        this.setAttributions();
        
        //
        this.events.triggerEvent('changelayer');
        
        //Legend
        //gt.control.legend.get();
        
    },
    
    add: function () {
        for(var i in this.nodes) {
            if(this.nodes[i].data.id == 'base') {
                continue;
            }
            
            this.records.push(this.nodes[i].data);
        }
         
        //Order
        function byOrder(a, b) {
            return a.order - b.order;
        }
        this.records.sort(byOrder).reverse();

        for(var i in this.records) {
            if(! this.records[i].layerClass) {
                continue;
            }

            //Javascript
            if(this.records[i].jsLayer == true) {
                gt.layer[this.records[i].layerClass] = new Giscuit.Layer[this.records[i].layerClass]();
                gt.map.addLayer(gt.layer[this.records[i].layerClass]);
                gt.layer[this.records[i].layerClass].setVisibility(false);

            //Tms
            } else {
                var extension = this.records[i].format;
                if(extension == 'png24') {
                    extension = 'png';
                }

                var layer = new OpenLayers.Layer.TMS(this.records[i].layerClass, gt.config.staticserverUrls, {
                    'order': this.records[i].order ,
                    'layername': this.records[i].layerClass ,
                    'transitionEffect': this.records[i].transitionEffect ,
                    'serviceVersion': '' ,
                    'type': extension ,
                    'visibility': false ,
                    'maxscaledenom': parseFloat(this.records[i].maxscaledenom),
                    'maxScale': parseFloat(this.records[i].minscaledenom),
                    'minscaledenom': parseFloat(this.records[i].minscaledenom),
                    'minScale': parseFloat(this.records[i].maxscaledenom),
                    'buffer': 1,
                    'isBaseLayer': false
                });

                gt.map.addLayer(layer);
                gt.layer[this.records[i].layerClass] = layer;
            }
        }
        
        gt.map.addVectorLayer();
    },
    
    addLayer: function(layer, record, type) {
        var maxId = 0;
        for (var i = 0; i < this.records.length; i++) {
            if (this.records[i].id > maxId) {
                maxId = this.records[i].id;
            }
        }
        maxId++;
        
        record.id = maxId;
        record.layerClass = layer.name;
        record.js_layer = true;
        record.sendRequest = false;
        record['class'] = layer.name;
        record.edit = 'none';
        record.label = 'none';
        
        if (typeof(record.metadata) == 'undefined') {
            record.metadata = {};
        }
        
        if (layer.visibility) {
            record.view = true;
        }
        
        this.records.push(record);
        gt.map.addLayer(layer);
        gt.layer[record.layerClass] = layer;
        
        if (type == 'base') {
            gt.map.setLayerIndex(layer, 0);
            var node = new YAHOO.widget.TextNode({label: record.name, data:record}, this.nodes['base'], false); 
        } else {
            var node = new YAHOO.widget.TextNode({label: record.name, data:record}, this.tree.getRoot(), false);
            node.insertBefore(this.tree.getRoot().children[0]);
        }
        
        if (layer.visibility) {
            node.highlight();
        }
        this.nodes[record.id] = node;
        this.tree.draw();
    },
    
    removeLayer: function(className) {

        for (var key in gt.layer) {
            if (key == className) {
                var layer = gt.layer[className];
                
                //Delete from gt.layer and map
                delete gt.layer[className];
                gt.map.removeLayer(layer);
                
                //Delete from records
                for (var i in this.records) {
                    if (this.records[i].layerClass == className) {
                        this.records.splice(i, 1);
                        break;
                    }
                }

                //Delete from tree
                for(var i in this.nodes) {
                    if (this.nodes[i].data.layerClass == className) {
                        this.tree.removeNode(this.nodes[i], true);
                        delete this.nodes[i];
                        break;
                    }
                }
                this.tree.draw();
                
                return;
            }
        }
    },

    setAttributions: function() {
        var div = YAHOO.util.Dom.get('gtAttribution');
        var attributions = [gt.config.copyRight];
        
        for(var obj in this.nodes) {
            var node = this.nodes[obj];
            
            if (node.highlightState == 1 && node.data.metadata) {
                var metadata = node.data.metadata;
                
                //Base
                if (metadata instanceof Array) {
                    for(var a=0; a < metadata.length; a++) {
                        
                        if(! metadata[a].ows_attribution_title) {
                            continue;
                        }
                        
                        var attribution = metadata[a].ows_attribution_title;
                        if(metadata[a].ows_attribution_onlineresource) {
                            attribution = "<a target='_blank' href='" + metadata[a].ows_attribution_onlineresource +  "'>" + attribution + "</a>";
                        }    
                        if(attribution && ! Giscuit.Util.inArray(attributions, attribution)) {
                            attributions.push(attribution);
                        }
                    }
                    
                //Overlay
                } else if(metadata.ows_attribution_title &&
                    ! Giscuit.Util.inArray(attributions, metadata.ows_attribution_title)) {
                    
                    var attribution = metadata.ows_attribution_title;
                    if(metadata.ows_attribution_onlineresource) {
                        attribution = "<a target='_blank' href='" + metadata.ows_attribution_onlineresource +  "'>" + attribution + "</a>";
                    }
                    if(attribution && ! Giscuit.Util.inArray(attributions, attribution)) {
                        attributions.push(attribution);
                    }

                }
            }
        }
        
        div.innerHTML = attributions.join(', ');
    },
    
    setSettings: function(settingsName, layerId, action) {
        if(settingsName != 'view') {
            return false;
        }
        
        var node = null; 
        for(var i in this.nodes) {
            if(this.nodes[i].data.id == layerId) {
                node = this.nodes[i];
                break;
            }
        }
        
        if(! node) {
            return false;
        }
        
        if(!!node.highlightState != action) {
            if(action) {
                node.highlight();
            } else {
                node.unhighlight();
            }
        }
        
        return true;
    },
    
    getSettings: function(settingsName, layerId) {
        if(settingsName != 'view') {
            return undefined;
        }
        
        var node = null; 
        for(var i in this.nodes) {
            if(this.nodes[i].data.id == layerId) {
                node = this.nodes[i];
                break;
            }
        }
        
        if(! node) {
            return undefined;
        }
        
        return !!node.highlightState;
    },
    
    getLayerIdByClassName: function(val) {
        for(var i in this.nodes) {
            if(this.nodes[i].data.layerClass == val) {
                return this.nodes[i].data.id;
            }
        }
        
        return false;
    },
    
    CLASS_NAME: "Giscuit.Control.Publiclayers"
});