/**
 * Giscuit

 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Info.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.quickaccess = new Giscuit.Mobile.Control.Quickaccess();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Quickaccess
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Quickaccess = OpenLayers.Class(Giscuit.Control, {
    
    initialize: function (options) {
        
        //
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtQuickaccessButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -896px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('quickaccess') + '</div>'
            + '</div></div>');
        
        // 
        $("#gtQuickaccessButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            //Hide map controls
            $("#gtMapControls").css('display', 'none');
            $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
            
            $.mobile.showPageLoadingMsg();
            
            //
            if(! $('#gtQuickaccess').length) {
                evt.data.render();
            }
            
            $.mobile.changePage($('#gtQuickaccess'), {
                'changeHash': true
            });
            
            $.mobile.hidePageLoadingMsg();
            
            return false;
        });
        
        if (gt.config.guest == 'f') {
            //Add quickaccess layer to restricted offline layers
            gt.control.offline.restrictedLayers.push('Quickaccess');
            
            //Register for create event
            gt.control.editgeom.events.register("addfeature", this, function(data) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
                
                this.addRecord(data);
                this.buildList();
            });
            
            //Register for update event
            gt.control.editgeom.events.register("editfeature", this, function(data) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
                
                this.deleteRecord(data.values.gid);
                this.addRecord(data);
                this.buildList();
            });
            
            //Register for delete event
            gt.control.editgeom.events.register("deletefeature", this, function(gid) {
                //Check the editable layer
                if (gt.control.layers.getEditLayer()['class'] != 'Quickaccess') {
                    return;
                }
            
                this.deleteRecord(gid);
                this.buildList();
            });
        }
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    render: function() {
        
        var html = '<div data-role="page" id="gtQuickaccess" data-dom-cache="true" data-url="gtQuickaccess">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('quickaccess') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
         
            '<div data-role="content">' + 
                '<ul id="gtQuickaccessRecords" data-role="listview">' +
                '</ul>' +
            '</div>' +        
            
        '</div>';
        
        $.mobile.pageContainer.append(html);
        $('#gtQuickaccess').page();
        
        this.buildList();

        //EVENTS:
        
        $("#gtQuickaccessRecords").on(gt.config.clickEvt, '.gtQuickaccessRecord', this, function(evt) {
            var geometryStr = $(this).jqmData('geometry');
            var geometry = OpenLayers.Geometry.fromWKT(geometryStr);
            gt.map.zoomToExtent(geometry.getBounds());
            
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });

    },
    
    addRecord: function (data) {
        var zoneName = data.values['name_' + gt.config.locale];
        var item = {
            gid: data.values.gid,
            name: zoneName,
            published: data.values.published,
            the_geom: data.feature.geometry.toString()
        };
        
        if (data.values.published == 't') {
            gt.pluginParams.quickaccessRecords.unshift(item);
        } else {
            gt.pluginParams.quickaccessRecords.push(item);
        }
    },
    
    deleteRecord: function(gid) {
        for (var i = 0; i < gt.pluginParams.quickaccessRecords.length; i++) {
            if (gt.pluginParams.quickaccessRecords[i].gid == gid) {
                gt.pluginParams.quickaccessRecords.splice(i, 1);
                break;
            }
        }
    },
    
    buildList: function() {
        if(! $('#gtQuickaccess').length) {
            return;
        }
        
        var html = '';
        var separatorAdded = false;
        for (var i in gt.pluginParams.quickaccessRecords) {
            if (gt.pluginParams.quickaccessRecords[i].published != 't' && ! separatorAdded) {
                separatorAdded = true;
                html += '<li data-role="list-divider"></li>';
            }
            html += '<li data-gid="' + gt.pluginParams.quickaccessRecords[i].gid + '" data-geometry="' + 
                gt.pluginParams.quickaccessRecords[i].the_geom + '" ' + 'class="gtQuickaccessRecord"><a href="">' + 
                gt.pluginParams.quickaccessRecords[i].name + '</a></li>';
        }
        
        $('#gtQuickaccessRecords').html(html).listview('refresh');
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Quickaccess"
});