/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 *
 *
 * @version    $Id: Search.js 1555 2011-08-02 15:41:37Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 *
 */

Giscuit.Map.prototype.gtEventListeners.push({
    'draw': function() {
        gt.control.search = new Giscuit.Mobile.Control.Search();
    }
});

/**
 * Class: Giscuit.Mobile.Control.Search
 * 
 *  Inherits from:
 *   - <Giscuit.Control>
 */
Giscuit.Mobile.Control.Search = OpenLayers.Class(Giscuit.Control, {
    
    selected: null,
    
    selectControl: null,
    
    limit: 100,
    
    initialize: function (options) {
        
        //
        $("#gtMapControls").append('<div>'
            + '<div class="ui-bar ui-bar-c" id="gtSearchButton">'
            + '<div class="gtMapControlsElImage" style="background-position: -192px 50%;"></div>'
            + '<div class="gtMapControlsElText">' + gt.translate._('search') + '</div>'
            + '</div></div>');
        
        //Activate
        $("#gtSearchButton").on(gt.config.clickEvt, this, function(evt) {
            //
            evt.stopImmediatePropagation();
            
            evt.data.activate();
            
            return false;
        });
        
        Giscuit.Control.prototype.initialize.apply(this, [options]);
    },
    
    render: function () {
        var form = '<form id="gtSearchForm" enctype="application/x-www-form-urlencoded" method="post" action="">' +
            '<dl class="zend_form">' +
            '<dt id="gtSearchLayer-label"><label for="gtSearchLayer" class="required">' + gt.translate._('layer') + '</label></dt>' +
                '<fieldsset data-role="controlgroup"> <select name="layer" id="gtSearchLayer">' +
                '</select></fieldsset>' +
            '<dt id="gtSearchField-label"><label for="gtSearchField" class="required">' + gt.translate._('field') + '</label></dt>' +
                '<fieldsset data-role="controlgroup"> <select name="field" id="gtSearchField">' +
                '</select></fieldsset>' +
            '<dt id="gtSearchWhere-label"><label for="gtSearchWhere" class="optional">' + gt.translate._('where') + '</label></dt>' +
                '<fieldsset data-role="controlgroup"> <label for="gtSearchWhere-everywhere">' +
                    '<input type="radio" name="where" id="gtSearchWhere-everywhere" value="everywhere" checked="checked">' + gt.translate._('everywhere') + '</label>' +
                    '<label for="gtSearchWhere-bbox"><input type="radio" name="where" id="gtSearchWhere-bbox" value="bbox">' + gt.translate._('withInCurrentBbox') + '</label>' +
                '</fieldsset>' +
            '<dt id="gtSearchText-label"><label for="gtSearchText" class="optional">' + gt.translate._('text') + '</label></dt>' +
            '<dd id="gtSearchText-element"> <input type="text" name="text" id="gtSearchText" value="" onkeypress="return event.keyCode != 13"></dd>' +
            '<button type="button" name="submit_button" id="gtSearchSubmit">' + gt.translate._('search') + '</button>' + 
        '</dl></form>';
        
        var html = '<div data-role="page" id="gtSearch" data-dom-cache="true" data-url="gtSearch">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('search') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
            '<div data-role="content">' + 
                form +
            '</div>' +    
        '</div>' +
        '<div data-role="page" id="gtSearchResult" data-dom-cache="true" data-url="gtSearchResult">' +
            '<div data-role="header">' +
                '<h1>' + gt.translate._('search') + '</h1>' +
                '<a href="#gtMap" data-icon="map" data-direction="reverse" class="ui-btn-right jqm-home">' +
                    gt.translate._('map') + 
                '</a>' + 
            '</div>' +
            '<div data-role="content">' +
                '<ul data-role="listview" id="gtSearchResultList"></ul>' +
            '</div>' +    
        '</div>';
        
        //
        $.mobile.pageContainer.append(html);
        $('#gtSearch').page();
        $('#gtSearchResult').page();
        
        //EVENTS:
        
        //Update field
        $('#gtSearchLayer').on('change', this, function(evt) {
            evt.data.updateField();
        });
        
        //Submit
        $('#gtSearchSubmit').on(gt.config.clickEvt, this, function(evt) {
            $(this).button('disable');
            $.mobile.showPageLoadingMsg();
            
            Giscuit.Util.bind(evt.data.submit(), evt.data);
        });
        
        //Show on map
        $(document).on(gt.config.clickEvt, '.gtSearchShowOnMapButton', this, function(evt) {
            var record = gt.control.layers.getLayerRecordById($('#gtSearchLayer').val());
            
            var geometry = $(this.parentNode.parentNode).jqmData('geometry');
            var layerClass = $(this.parentNode.parentNode).jqmData('layer-class');
            var featureId = $(this.parentNode.parentNode).jqmData('feature-id');
                        
            if(typeof(geometry) != 'undefined') {
                var geometry = OpenLayers.Geometry.fromWKT(
                    $(this.parentNode.parentNode).jqmData('geometry')
                );
                
                if(evt.data.selected != null) {
                    gt.layer.vector.destroyFeatures([evt.data.selected]);
                    evt.data.selected = null;
                }
                evt.data.selected = new OpenLayers.Feature.Vector(geometry);
                
                gt.layer.vector.addFeatures([evt.data.selected]);
                gt.map.zoomToExtent(geometry.getBounds());
            } else if (typeof(layerClass) != 'undefined' && typeof(featureId) != 'undefined') {
                var feature = gt.layer[layerClass].getFeatureByFid(featureId);
                
                evt.data.selectControl.unselectAll();
                evt.data.selectControl.select(feature);
                gt.map.zoomToExtent(feature.geometry.getBounds());
            }
            
            //Zoom to scale if available
            if(record.search_scale != null) {
                gt.map.zoomToScale(parseInt(record.search_scale));
            }
                        
            $.mobile.changePage($('#gtMap'), {
                'changeHash': true
            });
            
            return false;
        });
    },
    
    activate: function () {
        var activated = Giscuit.Control.prototype.activate.call(this);
        
        //Hide map controls
        $("#gtMapControls").css('display', 'none');
        $("#gtToggleControlsButton").css('bottom', '5px').removeClass('ui-btn-active');
                
        //
        if(! $('#gtSearch').length) {
            this.render();
        }
        
        this.updateLayer();
        this.updateField();
            
        $.mobile.changePage($('#gtSearch'));
        
        return activated;
    },
    
    deactivate: function () {
        var deactivated = Giscuit.Control.prototype.deactivate.call(this);
                
        //Destroy features
        if(this.selected != null) {
            gt.layer.vector.destroyFeatures([this.selected]);
            this.selected = null;
        } 
        
        if(this.selectControl != null) {
            this.selectControl.unselectAll();
            this.selectControl.destroy();
            this.selectControl = null;
        }
        
        return deactivated;
    },
    
    submit: function() {
        var record = gt.control.layers.getLayerRecordById($('#gtSearchLayer').val());
        
        //Online
        if (gt.control.offline.isOnline && ! record.js_layer) {
            var url = gt.config.httpMobilePath + '/search/index';

            //BBOX
            if($('#gtSearchWhere-bbox').is(':checked')) {
                $('#gtSearchWhere-bbox').val(gt.map.getExtent().toBBOX());
            }
            
            $.ajax({
                'type': 'POST',
                'url': url,
                'data': $('#gtSearchForm').serialize(),
                'success': this.set,
                'error': Giscuit.Util.jqmAjaxError ,
                'dataType': 'json' ,
                'context': this
            });
        
        //Offline
        } else {
            var layer = gt.layer[record['class']];
            var currentBounds = gt.map.getExtent();
            
            var field = $('#gtSearchField').val();
            var fieldValue = $('#gtSearchText').val().toString().toLowerCase();
            var searchEverywhere = $('#gtSearchWhere-everywhere')[0].checked;
            
            //Get default field
            if(record.default_field) {
                var defaultField = record.default_field;
            } else if(typeof(layer.defaultField) != 'undefined') {
                var defaultField = layer.defaultField;
            } else {
                for(var defaultField in layer.features[0].attributes) {
                    break;
                }
            }
           
            var data = {
                'defaultField': defaultField,
                'records': [],
                'fields': [],
                'limit': this.limit
            };
            
            //
            for (var i = 0; i < layer.features.length; i++) {
                if (layer.features[i].attributes[field] == null ||
                    data.records.length >= this.limit) {
                    
                    continue;
                }
                if (layer.features[i].attributes[field].toString().toLowerCase().search(fieldValue) != -1) {
                    var featureAttributes = layer.features[i].attributes;
                    if(gt.control.offline.isOnline) {
                        featureAttributes.the_geom = layer.features[i].geometry.toString();
                    } else {
                        featureAttributes.__layerClass = record['class'];
                        featureAttributes.__featureId = layer.features[i].fid;
                    }
                    
                    if(! searchEverywhere) {
                        if (currentBounds.intersectsBounds(layer.features[i].geometry.bounds)) {
                            data.records.push(featureAttributes);
                        }
                    } else {
                        data.records.push(featureAttributes);
                    }
                }
            }
            
            //
            for (var key in gt.map.removeReservedAttributes(layer.features[0].attributes)) {
                data.fields.push(key);
            }
            
            this.set(data);
            
        }
        
    },

    set: function (data) {
        //
        var html = '';
        
        //
        Giscuit.Util.jqmCleanPage('gtSearchResult');
                
        //Empty select result
        if(data.records.length == 0) {
            Giscuit.Util.jqmMessage(gt.translate._('emptyResult'));

            $('#gtSearchSubmit').button('enable');
            return;
        }

        for(var a = 0; a < data.records.length; a++) {
            var item = data.records[a];
            if(item[data.defaultField]) {
                //Check if default field is a file field
                if (typeof item[data.defaultField] == 'object') {
                    html += '<li>' + item[data.defaultField].value + '<ul>';
                } else {
                    html += '<li>' + item[data.defaultField] + '<ul>';
                }
            } else {
                html += '<li>' + gt.translate._('emptyField') + '<ul>';
            }
            
            //
            var bLen = (data.fields.length < gt.config.maxFieldsAmount) ? data.fields.length : gt.config.maxFieldsAmount;
            
            for(var b = 0; b < bLen; b++) {
                var field = data.fields[b];
                
                //
                if(field == '__layerClass' || field == '__featureId') {
                    continue;
                }
                
                if((typeof(item[field]) != 'object' && typeof(item[field]) != 'function') || 
                    item[field] === null) {
                    
                    if(item[field] === null) {
                        item[field] = '';
                    }

                    html += '<li>' + field + ': ' + item[field] + '</li>';
                    
                } else if(typeof(item[field]) == 'object' && 
                    item[field].type && 
                    (item[field].type == 'image' || item[field].type == 'file')
                    ) {
                    
                    html += '<li><a target="_blank" href="' +
                    item[field].url +
                    '">' +
                    field + ': ' + item[field].value +
                    '</a></li>';
                }
            }
            
            //
            if(typeof(item.the_geom) != 'undefined') {
                var jqmData = 'data-geometry="' + item.the_geom + '"';
            } else if(typeof(item.__layerClass) != 'undefined' && typeof(item.__featureId) != 'undefined') {
                var jqmData = 'data-layer-class="' + item.__layerClass + '" data-feature-id="' + item.__featureId + '"';
            }
            html += '<li ' + jqmData + '><button type="button" class="gtSearchShowOnMapButton" style="z-index: 1100;">' + gt.translate._('showResultsOnMap') + '</button></li>';

            html += '</ul></li>';
        }
        
        /*
        //Show column limitation tooltip
        if (maxLength == gt.config.maxFieldsAmount) {
            var firstNFieldsDisplayed = gt.translate._('firstNFieldsDisplayed').replace("%1$s", gt.config.maxFieldsAmount);
            Giscuit.Util.jqmMessage(firstNFieldsDisplayed);
        }
        */
            
        //
        $('#gtSearchResultList')
            .html(html)
            .listview('refresh');

        //
        if (data.records.length == 1) {
            $("div[data-role='page']").each(function (index, value) {
                var el = $(value);
                var page = 'gtSearchResult&ui-page=';
                if(el.jqmData('url').substr(0, page.length) == page) {
                    $.mobile.changePage('#' + el.jqmData('url'), {
                        'changeHash': true
                    });
                }
            });
        } else { 
            $.mobile.changePage($('#gtSearchResult'), {
                'changeHash': true
            });
        }
        
        //
        $('#gtSearchSubmit').button('enable');
        $.mobile.hidePageLoadingMsg();
    } ,
    
    updateLayer: function() {
        var records = gt.control.layers.records;
        var option = null;
        var select = $('#gtSearchLayer');
        
        select.children().remove();
        
        //Private
        if(gt.config.guest == 'f') {
            //Online
            if(gt.control.offline.isOnline) {
                for (var i = 0; i < records.length; i++) {
                    if (records[i].view && (records[i].fields.length || records[i].js_layer)) {
                        option = $('<option>')
                            .val(records[i].id)
                            .text(records[i].name);
                        
                        select.append(option);
                    }
                }
            //Offline
            } else {
                //
                var offlineLayers = [];
                
                //
                if(this.selectControl != null) {
                    this.selectControl.unselectAll();
                    this.selectControl.destroy();
                    this.selectControl = null;
                }
                
                //
                for (var i = 0; i < records.length; i++) {
                    if (! records[i].offline || ! records[i].view) {
                        continue;
                    }
                    
                    var layer = gt.layer[records[i]['class']];
                    
                    if(typeof(layer.features) == 'undefined') {
                        continue;
                    }
                    
                    //
                    offlineLayers.push(layer);
                    
                    //
                    option = $('<option>')
                        .val(records[i].id)
                        .text(records[i].name);
                    
                    select.append(option);
                    
                }
                
                //
                this.selectControl = new OpenLayers.Control.SelectFeature(offlineLayers);
                gt.map.addControl(this.selectControl);
                this.selectControl.activate();
                this.selectControl.handlers.feature.deactivate();
            }
        }
        
        //Public
        if(gt.config.guest == 't') {
            for (var i = 0; i < records.length; i++) {
                if (records[i].fields.length || records[i].js_layer) {
                    option = $('<option>')
                        .val(records[i].id)
                        .text(records[i].name);
                    
                    select.append(option);
                }
            }
        }
        
        select.selectmenu('refresh');
    },
    
    updateField: function () {
        //
        var option = null;
        var select = $('#gtSearchField');
          
        //
        select.children().remove();
        
        //
        var record = gt.control.layers.getLayerRecordById($('#gtSearchLayer').val());
        var layer = gt.layer[record['class']];
        
        //Online
        if(gt.control.offline.isOnline) {
            //JavaScript
            if(record.js_layer) {
                var attrs = gt.map.removeReservedAttributes(layer.features[0].attributes);
                var fields = [];
                for(var i in attrs) {
                    fields.push(i);
                }
                
                var defaultField = layer.defaultField
            
                //
            } else {
                var fields = record.fields;
                var defaultField = record.default_field;
            }
        //Offline
        } else {
            var attrs = gt.map.removeReservedAttributes(layer.features[0].attributes);
            var fields = [];
            for(var i in attrs) {
                fields.push(i);
            }

            var defaultField = record.default_field;
        }
                
        //
        for (var i in fields) {
            option = $('<option>')
                .val(fields[i])
                .text(gt.translate._(fields[i]));
            
            select.append(option);
        }
        select.val(defaultField);

        //
        select.selectmenu('refresh');
    },
    
    CLASS_NAME: "Giscuit.Mobile.Control.Search"
});
