/**
 * Giscuit
 *
 * LICENSE
 *
 * This source file is subject to the Giscuit license that is bundled
 * with this package in the file LICENSE.txt.
 *
 * @version    $Id: Util.js 3142 2012-11-14 15:13:45Z pavel.iacovlev $
 * @copyright  Copyright (c) 2012 VEC
 */
/**
 * Namespace: Giscuit.Util
 * 
 */

Giscuit.Util.insertAfter = function (referenceNode, newNode) {
    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
};

Giscuit.Util.inArray = function(arr, p_val) {
    for(var i = 0, l = arr.length; i < l; i++) {
        if(arr[i] == p_val) {
            return true;
        }
    }
    return false;
};

Giscuit.Util.unique = function (arr) {
    var r = new Array();
    o:for(var i = 0, n = arr.length; i < n; i++)
    {
        for(var x = 0, y = r.length; x < y; x++)
        {
            if(r[x] == arr[i])
            {
                continue o;
            }
        }
        r[r.length] = arr[i];
    }
    return r;
};

Giscuit.Util.isEmpty = function (obj) {
    for(var prop in obj) {
        if(obj.hasOwnProperty(prop))
            return false;
    }
    return true;
};

Giscuit.Util.getHash = function()
{
    var hash = window.location.hash;
    hash = hash.substring(1, hash.length);
    hash = hash.split("&");

    var result = new Object;
    for(var i = 0; i < hash.length; i++) {
        var item = hash[i].split("=");
        
        if(item[0] && item[1]) {
            result[item[0]] = item[1];
        }
    }
    return result;
};

Giscuit.Util.setHash = function(hash)
{
    var queryStr = '';
    for(var obj in hash) {
        queryStr += obj + '=' + hash[obj] + '&';
    }
    
    if(queryStr.charAt(queryStr.length - 1) == '&') {
        queryStr = queryStr.substring(0, queryStr.length - 1);
    }
    
    window.location.hash = queryStr;
    
    return queryStr;
};

Giscuit.Util.bind = function (func, object) {
    var args = Array.prototype.slice.apply(arguments, [2]);
    return function () {
        var newArgs = args.concat(
        Array.prototype.slice.apply(arguments, [0]));
        return func.apply(object, newArgs);
    };
};

Giscuit.Util.parseDate = function(string) {
    if(gt.config.dateFormat == 'iso8601') {
        var regexp = "([0-9]{4})(-([0-9]{2})(-([0-9]{2})" +
        "(T([0-9]{2}):([0-9]{2})(:([0-9]{2})(\.([0-9]+))?)?" +
        "(Z|(([-+])([0-9]{2}):([0-9]{2})))?)?)?)?";
        
        var d = string.match(new RegExp(regexp));
    
        var offset = 0;
        var date = new Date(d[1], 0, 1);

        if (d[3]) { date.setMonth(d[3] - 1); }
        if (d[5]) { date.setDate(d[5]); }
        if (d[7]) { date.setHours(d[7]); }
        if (d[8]) { date.setMinutes(d[8]); }
        if (d[10]) { date.setSeconds(d[10]); }
        if (d[12]) { date.setMilliseconds(Number("0." + d[12]) * 1000); }
        if (d[14]) {
            offset = (Number(d[16]) * 60) + Number(d[17]);
            offset *= ((d[15] == '-') ? 1 : -1);
        }

        offset -= date.getTimezoneOffset();
        
        var time = (Number(date) + (offset * 60 * 1000));
        
        var returnDate = new Date();
        returnDate.setTime(Number(time));
        
        return returnDate;

    } else {
        debug.error('Invalid date format (' + gt.config.dateFormat + ')');
    }
};

Giscuit.Util.callbackFailure = function(o) {
    debug.group("Response failure");
    debug.warn("Transaction id: " + o.tId); 
    debug.warn("HTTP status: " + o.status);
    debug.warn("Status code message: " + o.statusText);
    debug.groupEnd();
    
    if(o.status == 401) {
        Giscuit.Util.message('Access forbidden');
    }
};

Giscuit.Util.parseFailure = function(o, e) {
    debug.group(e.message);
    debug.warn(o.responseText);
    debug.groupEnd();
    
    Giscuit.Util.message("An application error has occurred. Please contact your Administrator.");
};

Giscuit.Util.invalidReplyCode = function (responseText) {
    debug.group("Status code: " + responseText.replyCode);
    debug.warn(responseText);
    debug.groupEnd();
    
    Giscuit.Util.message("An application error has occurred. Please contact your Administrator.");
};

Giscuit.Util.message = function (text) {
    alert(text);
};

Giscuit.Util.jqmMessage = function (text, timeout) {
    if(typeof(timeout) == 'undefined') {
        timeout = 5000;
    }
    
    $.mobile.showPageLoadingMsg("b", text, true);
    
    setTimeout(function() {
        $.mobile.hidePageLoadingMsg();
    }, timeout);
};

Giscuit.Util.websqlFailure = function (tx, evt) {
    Giscuit.Util.jqmMessage(evt.message);
    debug.warn(evt.message);
};

Giscuit.Util.jqmAjaxError = function (evt) {
    debug.group(evt.statusText);
    debug.warn(evt.responseText);
    debug.groupEnd();
    
    Giscuit.Util.jqmMessage(evt.statusText);
};

Giscuit.Util.getStorageObject = function(key) {
    var jsonFormat = new OpenLayers.Format.JSON();

    if(localStorage.getItem(key)) {
        return jsonFormat.read(localStorage.getItem(key));
    } else {
        return {};
    }
};

Giscuit.Util.setStorageObject = function(key, value) {
    var jsonFormat = new OpenLayers.Format.JSON();
    localStorage.setItem(key, jsonFormat.write(value));
};

Giscuit.Util.removeStorageObject = function(key) {
    localStorage.removeItem(key);
};

Giscuit.Util.setCookie = function (name, value, days) {
    if(gt.config.link == 't' || gt.config.print == 't') {
        return null;
    }
     
    if (days) {
        var date = new Date();
        date.setTime(date.getTime()+(days*24*60*60*1000));
        var expires = "; expires="+date.toGMTString();
    }
    else var expires = "";
    document.cookie = name+"="+value+expires+"; path=/";
};

Giscuit.Util.getCookie = function (name) {
    if(gt.config.link == 't' || gt.config.print == 't') {
        return null;
    }
    
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for(var i=0;i < ca.length;i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') c = c.substring(1,c.length);
        if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
    }
    return null;
};

Giscuit.Util.deleteCookie = function (name) {
    setCookie(name,"",-1);
};

Giscuit.Util.fireEvent = function(element,event) {
    if (document.createEvent) {
       var evt = document.createEvent("HTMLEvents");
       evt.initEvent(event, true, true );
       return !element.dispatchEvent(evt);
       
    } else {
       var evt = document.createEventObject();
       return element.fireEvent('on' + event, evt)
   }
};

Giscuit.Util.debounce = function (func, threshold, execAsap) {
    var timeout;
    return function debounced () {
        var obj = this, args = arguments;
        function delayed () {
            if (!execAsap)
                func.apply(obj, args);
            timeout = null; 
        };
 
        if (timeout)
            clearTimeout(timeout);
        else if (execAsap)
            func.apply(obj, args);
 
        timeout = setTimeout(delayed, threshold || 100); 
    };
};

Giscuit.Util.populateYuiMenu = function (button, values, selectedValue) {
    var menu = button.getMenu();

    menu.clearContent();
    menu.addItems(values);
    menu.render();
    
    //Selected
    var selectedItem = menu.getItem(0);
    
    var items = menu.getItems();
    for(var i=0; i < items.length; i++) {
        if(selectedValue == items[i].value) {
            selectedItem = items[i];
            break;
        } 
    }
            
    menu.activeItem = selectedItem;
    
    if(typeof(menu.activeItem) != 'undefined') {
        button.set("label", menu.activeItem.cfg.getProperty('text'));
    } else {
        button.set("label", '');
    }
};

Giscuit.Util.getScrollBarWidth = function () {
    var inner = document.createElement('p');
    inner.style.width = "100%";
    inner.style.height = "200px";
    
    var outer = document.createElement('div');
    outer.style.position = "absolute";
    outer.style.top = "0px";
    outer.style.left = "0px";
    outer.style.visibility = "hidden";
    outer.style.width = "200px";
    outer.style.height = "150px";
    outer.style.overflow = "hidden";
    outer.appendChild (inner);
    
    document.body.appendChild (outer);
    var w1 = inner.offsetWidth;
    outer.style.overflow = 'scroll';
    var w2 = inner.offsetWidth;
    if (w1 == w2) w2 = outer.clientWidth;
    
    document.body.removeChild (outer);
    
    return (w1 - w2);
};

Giscuit.Util.getMenuItemById = function (button, id) {
    var items = button.getMenu().getItems();

    for(var i=0; i < items.length; i++) {
        if(items[i].id == id) {
            return items[i];
        }
    }

    return false;
};

Giscuit.Util.jqmCleanPage = function(name) {
    $("div[data-role='page']").each(function (index, value) {
        var el = $(value);
        var page = name + '&ui-page=';
        if(el.jqmData('url').substr(0, page.length) == page) {
            el.remove();
        }            
    });
};

//Giscuit.Util.mobileOs, Giscuit.Util.isMobile
(function() {
    var a = navigator.userAgent;
    
    Giscuit.Util.mobileOs = '';
    Giscuit.Util.isMobile = false;
    Giscuit.Util.mobileOsVersion = 0;
    
    //
    if (a.indexOf('Opera Mobi') != -1) {
        Giscuit.Util.mobileOs = 'opera';
        Giscuit.Util.mobileOsVersion = parseFloat(opera.version());

    //
    } else if (a.indexOf('Firefox') != -1 && a.indexOf('Mobile') != -1) {
        Giscuit.Util.mobileOs = 'firefox';
        if(/Firefox[\/\s](\d+\.\d+)/.test(a)) {
            Giscuit.Util.mobileOsVersion = parseFloat(RegExp.$1);
        }    
        
    //
    } else if(a.indexOf('Android') != -1) {
        Giscuit.Util.mobileOs = 'android';
        Giscuit.Util.mobileOsVersion = parseFloat(a.slice(a.indexOf('Android') + 8));
    
    //
    } else if(a.indexOf('iPad') != -1) {
        Giscuit.Util.mobileOs = 'ipad';
        Giscuit.Util.mobileOsVersion = parseFloat(a.substr(a.indexOf('OS ') + 3, 3).replace( '_', '.' ));
        
    } else if(a.indexOf('iPhone') != -1 || a.indexOf('iPod') != -1) {
        Giscuit.Util.mobileOs = 'iphone';
        Giscuit.Util.mobileOsVersion = parseFloat(a.substr(a.indexOf('OS ') + 3, 3).replace( '_', '.' ));
    
    //
    } else if(a.indexOf('webOS') != -1) {
        Giscuit.Util.mobileOs = 'webos';
    
    //
    } else if(a.indexOf('BlackBerry') != -1) {
        Giscuit.Util.mobileOs = 'blackberry';
    
    //
    } else if(a.indexOf('Windows Phone') != -1) {
        Giscuit.Util.mobileOs = 'windowsphone';
    }
    
    if(Giscuit.Util.mobileOs) {
        Giscuit.Util.isMobile = true;
    }
})();

Giscuit.Util.cogoAngleDistance = function (point, dist, angle) {
    var newX = point.x + dist * Math.sin(Giscuit.Util.deg2rad(angle));
    var newY = point.y + dist * Math.cos(Giscuit.Util.deg2rad(angle));
    
    return new OpenLayers.Geometry.Point(newX, newY);
};

Giscuit.Util.rad2deg = function (angle) {
    return angle / Math.PI * 180; 
};

Giscuit.Util.deg2rad = function (angle) {
    return angle / 180 * Math.PI; 
};