<?php

class PrivateLayersControllerTest extends Giscuit_Test_PHPUnit_ControllerTestCase
{
    protected $_layerName = 'Worldboundaries';
    
    protected $_secondLayerName = 'Rastercut';
    
    protected $_newLabelParams = array (
        'label_size' => '11',
        'label_color' => '#000000',
        'label_fields' => 'name'
    );
    
    protected $_defLabelParams = array (
        'label_size' => '10',
        'label_color' => '#000000',
        'label_fields' => 'name'
    );
    
    protected function _getIdFromClass($layer)
    {
        $layersTable = new Giscuit_Db_Table_Layers();                
        $row = $layersTable->getItems(array('whereClass' => $layer));
        foreach($row as $r) {
            $id = $r['id'];
        }        
        
        return $id;
    }
    
    protected function _getRowFromId($id, array $fields = array() )
    {
        $layersSettingsTable = new Giscuit_Db_Table_LayerSettings();
        $config = Zend_Registry::get('config');
        $userId = Zend_Controller_Action_HelperBroker::getStaticHelper('userId')->userId();

        $select = $layersSettingsTable->select();
        $select->from($config->database->schema . '.layer_settings', $fields);
        $select->where('layer_settings.layer_id = ?', $id);
        $select->where('layer_settings.user_id = ?', $userId);
        
        return $layersSettingsTable->fetchAll($select)->current();
    }
    
    protected function _updateParameter($param, $value)
    {
        $id = $this->_getIdFromClass($this->_layerName);
        
        $url = '/' . $this->_language . '/tool/privatelayers/edit/id/' . $id . '/column/' . $param . '/value/' . $value;
        $this->dispatch($url);
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        
        $this->clear();
    }
    
    public function testGetPublic()
    {
    	$id = $this->_getIdFromClass($this->_layerName);
    	
        $lsTable = new Giscuit_Db_Table_LayerSettings();
        
        $result = $lsTable->getPublicItems();

        $this->assertArrayHasKey($id, $result['layers']);
        $this->assertFalse($result['isEdit']);
    }
    
    public function testGetPrivate()
    {
    	$id = $this->_getIdFromClass($this->_layerName);
    	
        $lsTable = new Giscuit_Db_Table_LayerSettings();
        
        $this->_testLoginActionSuccess();
        
        $result = $lsTable->getPrivateItems();
        
        $this->assertArrayHasKey($id, $result['layers']);
    }

    public function testUpdateViewAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->_updateParameter('view', 'false');
    }
    
    public function testColumnAfterUpdateAndSetBack()
    {
        $this->_testLoginActionSuccess();
        
        $id = $this->_getIdFromClass($this->_layerName);
        $result = $this->_getRowFromId($id, array('view'));
        $this->assertEquals($result->view, 'f');
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/edit/id/' . $id . '/column/view/value/true');
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
    }
    
    public function testLabelAction()
    {
        $this->_testLoginActionSuccess();
        
        $id = $this->_getIdFromClass($this->_layerName);

        $this->dispatch('/' . $this->_language . '/tool/privatelayers/settings/id/' . $id);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertContains('gtPrivatelayersSettingsForm', $response['form']);
    }
    
    public function testLabelSubmitAction()
    {
        $this->_testLoginActionSuccess();
        
        $id = $this->_getIdFromClass($this->_layerName);

        $this->request->setMethod('POST')->setPost(array(
            'label_size' => $this->_newLabelParams['label_size'] ,
            'label_color' => $this->_newLabelParams['label_color'] ,
            'label_fields' => $this->_newLabelParams['label_fields']
        ));
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/settings/id/' . $id);
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);

        $this->assertEquals($response['replyCode'], 200);
    }

    public function testLabelAfterSubmitAndSetBack()
    {
        $this->_testLoginActionSuccess();
        
        $id = $this->_getIdFromClass($this->_layerName);
        $result = $this->_getRowFromId($id, array('label_size'));

        $this->assertEquals($result->label_size, intval($this->_newLabelParams['label_size']));
        
        $this->request->setMethod('POST')->setPost(array(
            'label_size' => $this->_defLabelParams['label_size'] ,
            'label_color' => $this->_defLabelParams['label_color'] ,
            'label_fields' => $this->_defLabelParams['label_fields']
        ));
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/settings/id/' . $id);
    } 
    
    /*
    public function testMoveAction()
    {
        $this->_testLoginActionSuccess();
        
        $firstId = $this->_getIdFromClass($this->_layerName);
        $secondId = $this->_getIdFromClass($this->_secondLayerName);
        
        $firstLayerOrder = $this->_getRowFromId($firstId, array('order'));
        $secondLayerOrder = $this->_getRowFromId($secondId, array('order'));

        $this->dispatch('/' . $this->_language . '/tool/privatelayers/move/first/' . $firstId . '/second/' . $secondId);
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);

        $this->assertEquals($response['replyCode'], 200);
        $this->assertEquals($response['selected'], $secondId);
        
        $movedfLayerOrder = $this->_getRowFromId($firstId, array('order'));
        $movedsLayerOrder = $this->_getRowFromId($secondId, array('order'));
        
        $this->assertEquals($firstLayerOrder->order, $movedsLayerOrder->order);
        $this->assertEquals($secondLayerOrder->order, $movedfLayerOrder->order);
    }
    
    public function testSetBackOrder()
    {
        $this->_testLoginActionSuccess();

        $firstId = $this->_getIdFromClass($this->_layerName);
        $secondId = $this->_getIdFromClass($this->_secondLayerName);
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/move/first/' . $firstId . '/second/' . $secondId);
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        $this->assertEquals($response['replyCode'], 200);
        $this->assertEquals($response['selected'], $secondId);
    }
    */
    
    public function testEditEnableAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->_updateParameter('edit', 'true');
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/editenable');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        
        $this->assertEquals($response['replyCode'], 200);
        $this->assertEquals($response['type'], 'Polygon');
        $this->assertTrue($response['editgeom']);
        $this->assertFalse($response['editattr']);
    }
    
    public function testEditDisableAction()
    {
        $this->_testLoginActionSuccess();
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/editdisable');
        
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);
        
        $this->assertEquals($response['replyCode'], 200);
        $this->assertFalse(Zend_Registry::get('settings')->editingActive);
        
        $this->clear();
        
        $this->_updateParameter('edit', 'false');
    }
    
    public function testOrderAction()
    {
    	$this->_testLoginActionSuccess();
    	
        $lsTable = new Giscuit_Db_Table_LayerSettings();
        $result = $lsTable->getPrivateItems();
        $lastIndex = count($result['layers']) - 1;

        $firstId = $result['layers'][0]['id'];
        $lastId = $result['layers'][$lastIndex]['id'];
        
        foreach ($result['layers'] as $layer) {
        	$newOrder[] = $layer['id'];
        }
        
        $newOrder[0] = $lastId;
        $newOrder[$lastIndex] = $firstId;
        
        $this->dispatch('/' . $this->_language . '/tool/privatelayers/order/ids/' . implode(',', $newOrder));
        $response = $this->getResponse()->getBody();
        $response = Zend_Json::decode($response);

        $this->assertEquals($response['replyCode'], 200);

        $result = $lsTable->getPrivateItems();
        
        $this->assertEquals($result['layers'][0]['id'], $lastId);
        $this->assertEquals($result['layers'][$lastIndex]['id'], $firstId);
    }
    
}